@@ -1,15 +1,12 @@
 module Main where
 import Text.CSL.Input.Bibutils (readBiblioString, BibFormat(..))
-import Text.CSL.Style (Formatted(..), Agent(..))
 import Text.CSL.Reference (Reference(refId), Literal(..))
-import Data.Generics ( everywhere, mkT )
 import Data.List (group, sort)
 import Data.Char (chr, toLower)
-import Data.Monoid
-import Data.Yaml
-import Control.Applicative
+import Data.Yaml.Builder (toByteString)
+import Control.Applicative ((<|>), many)
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Lazy.Char8 as BL8
+import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString.Char8 as B8
 import Data.Attoparsec.ByteString.Char8 as Attoparsec
 import qualified Data.Text as T
@@ -26,6 +23,7 @@ import Paths_pandoc_citeproc (version)
 import Text.CSL.Pandoc (processCites')
 import Text.Pandoc.JSON hiding (Format)
 import Text.Pandoc.Walk
+import qualified Text.Pandoc.UTF8 as UTF8
 
 main :: IO ()
 main = do
@@ -33,13 +31,13 @@ main = do
   let (flags, args, errs) = getOpt Permute options argv
   let header = "Usage: pandoc-citeproc [options] [file..]"
   unless (null errs) $ do
-    hPutStrLn stderr $ usageInfo (unlines $ errs ++ [header]) options
+    UTF8.hPutStrLn stderr $ usageInfo (unlines $ errs ++ [header]) options
     exitWith $ ExitFailure 1
   when (Version `elem` flags) $ do
-    putStrLn $ "pandoc-citeproc " ++ showVersion version
+    UTF8.putStrLn $ "pandoc-citeproc " ++ showVersion version
     exitWith ExitSuccess
   when (Help `elem` flags) $ do
-    putStrLn $ usageInfo header options
+    UTF8.putStrLn $ usageInfo header options
     exitWith ExitSuccess
   if Bib2YAML `elem` flags || Bib2JSON `elem` flags
      then do
@@ -50,20 +48,21 @@ main = do
                          msum (map formatFromExtension args) of
                          Just f   -> return f
                          Nothing  -> do
-                            hPutStrLn stderr $ usageInfo
+                            UTF8.hPutStrLn stderr $ usageInfo
                               ("Unknown format\n" ++ header) options
                             exitWith $ ExitFailure 3
        bibstring <- case args of
-                         []    -> getContents
-                         xs    -> mconcat <$> mapM readFile xs
+                         []    -> UTF8.getContents
+                         xs    -> mconcat <$> mapM UTF8.readFile xs
        readBiblioString bibformat bibstring >>=
          warnDuplicateKeys >>=
          if Bib2YAML `elem` flags
-            then outputYamlBlock . unescapeTags . encode
-            else BL8.putStrLn .
+            then outputYamlBlock .
+                 B8.intercalate (B.singleton 10) .
+                 map (unescapeTags . toByteString . (:[]))
+            else B8.putStrLn . unescapeUnicode . B.concat . BL.toChunks .
               encodePretty' Config{ confIndent = 2
-                                  , confCompare = compare } .
-              everywhere (mkT compressName)
+                                  , confCompare = compare }
      else toJSONFilter doCites
 
 formatFromExtension :: FilePath -> Maybe BibFormat
@@ -85,6 +84,7 @@ readFormat = go . map toLower
         go "copac"    = Just Copac
         go "json"     = Just Json
         go "mods"     = Just Mods
+        go "yaml"     = Just Yaml
         go _          = Nothing
 
 
@@ -92,7 +92,7 @@ doCites :: Pandoc -> IO Pandoc
 doCites doc = do
   doc' <- processCites' doc
   let warnings = query findWarnings doc'
-  mapM_ (hPutStrLn stderr) warnings
+  mapM_ (UTF8.hPutStrLn stderr) warnings
   return doc'
 
 findWarnings :: Inline -> [String]
@@ -117,42 +117,15 @@ options =
 
 warnDuplicateKeys :: [Reference] -> IO [Reference]
 warnDuplicateKeys refs = mapM_ warnDup dupKeys >> return refs
-  where warnDup k = hPutStrLn stderr $ "biblio2yaml: duplicate key " ++ k
+  where warnDup k = UTF8.hPutStrLn stderr $ "biblio2yaml: duplicate key " ++ k
         allKeys   = map (unLiteral . refId) refs
         dupKeys   = [x | (x:_:_) <- group (sort allKeys)]
 
 outputYamlBlock :: B.ByteString -> IO ()
 outputYamlBlock contents = do
-  putStrLn "---\nreferences:"
+  UTF8.putStrLn "---\nreferences:"
   B.putStr contents
-  putStrLn "..."
-
--- Compress particles and suffixes into given and last name,
--- as zotero JSON expects.  (We might also want to set parse-names = true.)
-compressName :: Agent -> Agent
-compressName ag = Agent{
-    givenName       = gn
-  , familyName      = fn
-  , droppingPart    = mempty
-  , nonDroppingPart = mempty
-  , literal         = literal ag
-  , nameSuffix      = mempty
-  , commaSuffix     = False
-  , parseNames      = True
-  }
-  where
-  spcat (Formatted []) y = y
-  spcat y (Formatted []) = y
-  spcat x y = x <> Formatted [Space] <> y
-  gnbase = givenName ag ++ [droppingPart ag | droppingPart ag /= mempty]
-  gn = case (gnbase, nameSuffix ag) of
-             ([], _)            -> []
-             (xs, Formatted []) -> xs
-             (xs, ns)           -> init xs ++
-                [last xs <> Formatted [if commaSuffix ag
-                                          then Str ",!"
-                                          else Str ",", Space], ns]
-  fn = spcat (nonDroppingPart ag) (familyName ag)
+  UTF8.putStrLn "..."
 
 -- turn
 -- id: ! "\u043F\u0443\u043D\u043A\u04423"
@@ -163,6 +136,11 @@ unescapeTags bs = case parseOnly (many $ tag <|> other) bs of
                        Left e  -> error e
                        Right r -> B.concat r
 
+unescapeUnicode :: B.ByteString -> B.ByteString
+unescapeUnicode bs = case parseOnly (many other) bs of
+                       Left e  -> error e
+                       Right r -> B.concat r
+
 tag :: Attoparsec.Parser B.ByteString
 tag = do
   _ <- string $ B8.pack ": ! "
