@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, FlexibleContexts #-}
+{-# LANGUAGE PatternGuards, FlexibleContexts, ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Eval
@@ -21,15 +21,16 @@ module Text.CSL.Eval
     ) where
 
 import Control.Arrow
-import Control.Applicative ( (<$>) )
 import Control.Monad.State
-import Data.Monoid (mempty, Any(..))
+import Data.Monoid (Any(..))
 import Data.Char ( toLower, isDigit, isLetter )
 import Data.Maybe
 import Data.String ( fromString )
-import Text.Pandoc.Definition (Inline(Str, Space, Link))
+import Text.Pandoc.Definition (Inline(Str, Space, Link), nullAttr)
 import Text.Pandoc.Walk (walk)
 import Text.Pandoc.Shared (stringify)
+import qualified Data.Text as T
+import Data.Maybe (isNothing)
 
 import Text.CSL.Eval.Common
 import Text.CSL.Eval.Output
@@ -38,24 +39,22 @@ import Text.CSL.Eval.Names
 import Text.CSL.Output.Plain
 import Text.CSL.Reference
 import Text.CSL.Style hiding (Any)
-import Text.CSL.Util ( readNum, last', proc, proc', query, betterThan )
+import Text.CSL.Util ( readNum, last', proc, proc', query, betterThan,
+                       safeRead )
 
 -- | Produce the output with a 'Layout', the 'EvalMode', a 'Bool'
 -- 'True' if the evaluation happens for disambiguation purposes, the
 -- 'Locale', the 'MacroMap', the position of the cite and the
 -- 'Reference'.
 evalLayout :: Layout   -> EvalMode -> Bool -> [Locale] -> [MacroMap]
-           -> [Option] -> Abbreviations -> Reference -> [Output]
-evalLayout (Layout _ _ es) em b l m o a r
+           -> [Option] -> Abbreviations -> Maybe Reference -> [Output]
+evalLayout (Layout _ _ es) em b l m o a mbr
     = cleanOutput evalOut
     where
       evalOut = case evalState job initSt of
-                  [] -> if isSorting em
-                        then []
-                        else [noOutputError]
-                  x | title r == Formatted [Str (citeId cit), Space, Str "not", Space,
-                                   Str "found!"]             -> [noBibDataError $ cit]
-                    | otherwise                              -> suppTC x
+                  x | isNothing mbr -> [noBibDataError cit]
+                    | null x        -> []
+                    | otherwise     -> suppTC x
       locale = case l of
                  [x] -> x
                  _   -> Locale [] [] [] [] []
@@ -64,22 +63,24 @@ evalLayout (Layout _ _ es) em b l m o a r
                  EvalCite    c -> c
                  EvalSorting c -> c
                  EvalBiblio  c -> c
-      initSt = EvalState (mkRefMap r) (Env cit (localeTerms locale) m
+      initSt = EvalState (mkRefMap mbr) (Env cit (localeTerms locale) m
                          (localeDate locale) o [] a) [] em b False [] [] False [] [] []
       suppTC = let getLang = take 2 . map toLower in
-               case (getLang $ localeLang locale, getLang $ unLiteral $ language r) of
-                 (_,  "en") -> id
-                 ("en", []) -> id
-                 _          -> proc' rmTitleCase
+               case (getLang $ localeLang locale,
+                     getLang . unLiteral . language <$> mbr) of
+                 (_,  Just "en") -> id
+                 (_,  Nothing)   -> id
+                 ("en", Just "") -> id
+                 _               -> proc' rmTitleCase
 
 evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] ->
-               [Sort] -> Abbreviations -> Reference -> [Sorting]
-evalSorting m l ms opts ss as r
+               [Sort] -> Abbreviations -> Maybe Reference -> [Sorting]
+evalSorting m l ms opts ss as mbr
     = map (format . sorting) ss
     where
       render       = renderPlain . formatOutputList
       format (s,e) = applaySort s . render $ uncurry eval e
-      eval     o e = evalLayout (Layout emptyFormatting [] [e]) m False l ms o as r
+      eval     o e = evalLayout (Layout emptyFormatting [] [e]) m False l ms o as mbr
       applaySort c s
           | Ascending {} <- c = Ascending  s
           | otherwise         = Descending s
@@ -211,11 +212,24 @@ evalElement el
                              "page"        -> getStringVar "page" >>= formatRange fm
                              "locator"     -> getLocVar >>= formatRange fm . snd
                              "url"         -> getStringVar "url" >>= \k ->
-                                              if null k then return [] else return [Output [OPan [Link [Str k] (k,"")]] fm]
-                             "doi"         -> getStringVar "doi" >>= \d ->
+                                              if null k then return [] else return [Output [OPan [Link nullAttr [Str k] (k,"")]] fm]
+                             "doi"         -> do d <- getStringVar "doi"
+                                                 let (prefixPart, linkPart) = T.breakOn (T.pack "http") (T.pack (prefix fm))
+                                                 let u = if T.null linkPart
+                                                            then "https://doi.org/" ++ d
+                                                            else T.unpack linkPart ++ d
+                                                 if null d
+                                                    then return []
+                                                    else return [Output [OPan [Link nullAttr [Str (T.unpack linkPart ++ d)] (u, "")]]
+                                                          fm{ prefix = T.unpack prefixPart, suffix = suffix fm }]
+                             "pmid"        -> getStringVar "pmid" >>= \d ->
                                               if null d
                                                  then return []
-                                                 else return [Output [OPan [Link [Str d] ("http://dx.doi.org/" ++ d, "")]] fm]
+                                                 else return [Output [OPan [Link nullAttr [Str d] ("http://www.ncbi.nlm.nih.gov/pubmed/" ++ d, "")]] fm]
+                             "pmcid"       -> getStringVar "pmcid" >>= \d ->
+                                              if null d
+                                                 then return []
+                                                 else return [Output [OPan [Link nullAttr [Str d] ("http://www.ncbi.nlm.nih.gov/pmc/articles/" ++ d, "")]] fm]
                              _             -> gets (env >>> options &&& abbrevs) >>= \(opts,as) ->
                                               getVar [] (getFormattedValue opts as f fm s) s >>= \r ->
                                               consumeVariable s >> return r
@@ -328,9 +342,9 @@ formatNumber f fm v n
       roman     = foldr (++) [] . reverse . map (uncurry (!!)) . zip romanList .
                   map (readNum . return) . take 4 . reverse
       romanList = [[ "", "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix" ]
-	          ,[ "", "x", "xx", "xxx", "xl", "l", "lx", "lxx", "lxxx", "xc" ]
-	          ,[ "", "c", "cc", "ccc", "cd", "d", "dc", "dcc", "dccc", "cm" ]
-	          ,[ "", "m", "mm", "mmm", "mmmm", "mmmmm"]
+                  ,[ "", "x", "xx", "xxx", "xl", "l", "lx", "lxx", "lxxx", "xc" ]
+                  ,[ "", "c", "cc", "ccc", "cd", "d", "dc", "dcc", "dccc", "cm" ]
+                  ,[ "", "m", "mm", "mmm", "mmmm", "mmmmm"]
                   ]
 
 
@@ -393,67 +407,58 @@ formatRange fm p = do
       joinRange (a, []) = a
       joinRange (a,  b) = a ++ "-" ++ b
 
-      process = case opt of
-                 "expanded" -> checkRange ts . printNumStr . map (joinRange . uncurry expandedRange)
-                 "chicago"  -> checkRange ts . printNumStr . map (joinRange . uncurry chicagoRange )
-                 "minimal"  -> checkRange ts . printNumStr . map (joinRange . uncurry minimalRange )
-                 _          -> checkRange ts . printNumStr . map (joinRange)
+      process = checkRange ts . printNumStr . case opt of
+                 "expanded" -> map (joinRange . expandedRange)
+                 "chicago"  -> map (joinRange . chicagoRange )
+                 "minimal"  -> map (joinRange . minimalRange 1)
+                 "minimal-two"  -> map (joinRange . minimalRange 2)
+                 _          -> map joinRange
   return [flip OLoc fm $ [OStr (process pages) emptyFormatting]]
 
-expandedRange :: String -> String -> (String, String)
-expandedRange sa [] = (sa,[])
-expandedRange sa sb = (p ++ reverse nA', reverse nB')
-    where
-      (nA,pA) = reverse >>> break isLetter >>> reverse *** reverse $ sa
-      (nB,pB) = reverse >>> break isLetter >>> reverse *** reverse $ sb
-      zipNum x y = zipWith (\a b -> if b == '+' then (a,a) else (a,b))
-                           (reverse x ++ take 10 (repeat '*'))
-                   >>> unzip >>> filter (/= '*') *** filter (/= '*') $
-                   (reverse y ++ repeat '+')
-      checkNum a b = let a' = take (length b) a
-                     in  readNum a' > readNum b
-      (p,(nA',nB'))
-          = case () of
-              _ | pA /= []
-                , checkNum nA nB       -> (,) [] $ (reverse $ pA ++ nA, reverse $ pB ++ nB)
-                | pA /= pB
-                , last' pA == last' pB -> (,) pA $ second (flip (++) (last' pA)) $ zipNum nA nB
-                | pA == pB             -> (,) pA $ second (flip (++) (last' pA)) $ zipNum nA nB
-                | pB == []             -> (,) pA $ second (flip (++) (last' pA)) $ zipNum nA nB
-                | otherwise            -> (,) [] $ (reverse $ pA ++ nA, reverse $ pB ++ nB)
+-- Abbreviated page ranges are expanded to their non-abbreviated form:
+-- 42–45, 321–328, 2787–2816
+expandedRange :: (String, String) -> (String, String)
+expandedRange (sa, []) = (sa,[])
+expandedRange (sa, sb)
+  | length sb < length sa =
+      case (safeRead sa, safeRead sb) of
+           -- check to make sure we have regular numbers
+           (Just (_ :: Int), Just (_ :: Int)) ->
+             (sa, take (length sa - length sb) sa ++ sb)
+           _ -> (sa, sb)
+  | otherwise = (sa, sb)
 
-minimalRange :: String -> String -> (String, String)
-minimalRange sa sb
-    = res
-    where
-      (a,b) = expandedRange sa sb
-      res   = if length a == length b
-              then second (filter (/= '+')) $ unzip $ doit a b
-              else (a,b)
-      doit (x:xs) (y:ys) = if x == y
-                           then (x,'+') : doit xs ys
-                           else zip (x:xs) (y:ys)
-      doit _      _      = []
+-- All digits repeated in the second number are left out:
+-- 42–5, 321–8, 2787–816.  The minDigits parameter indicates
+-- a minimum number of digits for the second number; thus, with
+-- minDigits = 2, we have 328-28.
+minimalRange :: Int -> (String, String) -> (String, String)
+minimalRange minDigits ((a:as), (b:bs))
+  | a == b
+  , length as == length bs
+  , length bs >= minDigits =
+                let (_, bs') = minimalRange minDigits (as, bs)
+                in  (a:as, bs')
+minimalRange _ (as, bs) = (as, bs)
 
-chicagoRange :: String -> String -> (String, String)
-chicagoRange sa sb
-    = case () of
-        _ | length sa < 3    -> expandedRange sa sb
-          | '0':'0':_ <- sa' -> expandedRange sa sb
-          | _  :'0':_ <- sa' -> minimalRange  sa sb
-          | _  :a2:as <- sa'
-          , b1 :b2:bs <- sb'
-          , comp as bs       -> if a2 == b2
-                                then (sa, [b2,b1])
-                                else minimalRange sa sb
-
-          | _:a2:a3:_:[] <- sa'
-          , _:b2:b3:_    <- sb' -> if a3 /= b3 && a2 /= b2
-                                   then expandedRange sa sb
-                                   else minimalRange  sa sb
-          | otherwise           -> minimalRange sa sb
-      where
-        sa' = reverse sa
-        sb' = reverse sb
-        comp a b = let b' = takeWhile isDigit b
-                   in take (length b') a == b'
+-- Page ranges are abbreviated according to the Chicago Manual of Style-rules:
+-- First number             Second number    Examples
+-- Less than 100            Use all digits   3–10; 71–72
+-- 100 or multiple of 100   Use all digits   100–104; 600–613; 1100–1123
+-- 101 through 109 (in multiples of 100) Use changed part only  10002-6, 505-17
+-- 110 through 199          Use 2 digits or more  321-25, 415-532
+-- if numbers are 4 digits long or more and 3 digits change, use all digits
+--         1496-1504
+chicagoRange :: (String, String) -> (String, String)
+chicagoRange (sa, sb)
+    = case (safeRead sa :: Maybe Int) of
+          Just n | n < 100 -> expandedRange (sa, sb)
+                 | n `mod` 100 == 0 -> expandedRange (sa, sb)
+                 | n >= 1000 -> let (sa', sb') = minimalRange 1 (sa, sb)
+                                in  if length sb' >= 3
+                                       then expandedRange (sa, sb)
+                                       else (sa', sb')
+                  | n > 100 -> if n `mod` 100 < 10
+                                 then minimalRange 1 (sa, sb)
+                                 else minimalRange 2 (sa, sb)
+          _ -> expandedRange (sa, sb)
