@@ -15,13 +15,11 @@
 
 module Text.CSL.Eval.Names where
 
-import Control.Applicative ( (<$>) )
 import Control.Monad.State
 import Data.Char  ( isLower, isUpper, isLetter )
 import Data.List  ( nub, intersperse )
 import Data.List.Split ( wordsBy )
 import Data.Maybe ( isJust )
-import Data.Monoid
 
 import Text.CSL.Eval.Common
 import Text.CSL.Eval.Output
@@ -190,7 +188,7 @@ formatNames ea del p s as n
                           case getOptionVal "delimiter-precedes-et-al" o of
                             "never"  -> return . (++) [OSpace] $ output fm x
                             "always" -> return . (++) [ODel d] $ output fm x
-                            _        -> if i > 1
+                            _        -> if i > 1 && not (null d)
                                         then return . (++) [ODel d] $ output fm x
                                         else return . (++) [OSpace] $ output fm x
 
@@ -301,7 +299,7 @@ formatName m b f fm ops np n
       -- see src/load.js ROMANESQUE_REGEX in citeproc-js:
       isByzantine c = not (isLetter c) ||
                       c <= '\x5FF' ||
-                      (c >= '\x1f00' && c <= '\x1fff')
+                      (c >= '\x1e00' && c <= '\x1fff')
       shortName = oPan' (unFormatted $ nondropping <+> family) (form "family")
       longName g = if isSorting m
                    then let firstPart = case getOptionVal "demote-non-dropping-particle" ops of
@@ -378,6 +376,7 @@ s  <+> Formatted [] = s
 Formatted xs <+> Formatted ys =
   case lastInline xs of
        "’"  -> Formatted (xs ++ ys)
+       "-"  -> Formatted (xs ++ ys)
        _    -> Formatted (xs ++ [Space] ++ ys)
 
 (<++>) :: [Output] -> [Output] -> [Output]
