@@ -21,7 +21,6 @@ import Data.Char (toLower, toUpper)
 import Text.CSL.Util (capitalize, titlecase, unTitlecase, isPunct)
 import Text.Pandoc.Definition
 import Text.Pandoc.Walk (walk)
-import Data.Monoid (mempty, mconcat, (<>))
 import Data.String (fromString)
 import Data.Maybe (mapMaybe)
 
@@ -116,7 +115,7 @@ formatOutput o =
                                      [Strong [Str "???"]]]
       OLabel   []      _  -> Formatted []
       OLabel   s       f  -> formatOutput (OStr s f)
-      ODate    os         -> format os
+      ODate    os         -> formatOutputList os
       OYear    s _     f  -> formatOutput (OStr s f)
       OYearSuf s _ _   f  -> formatOutput (OStr s f)
       ONum     i       f  -> formatOutput (OStr (show i) f)
@@ -124,17 +123,19 @@ formatOutput o =
                                 then Formatted [Strong [Str "???"]]
                                 else formatOutput (OStr (show i) f)
       OName  _ os _    f  -> formatOutput (Output os f)
-      OContrib _ _ os _ _ -> format os
+      OContrib _ _ os _ _ -> formatOutputList os
       OLoc     os      f  -> formatOutput (Output os f)
       Output   []      _  -> Formatted []
-      Output   os      f  -> addFormatting f $ format os
+      Output   os      f  -> addFormatting f $ formatOutputList os
       _                   -> Formatted []
-    where
-      format = mconcat . map formatOutput
 
 addFormatting :: Formatting -> Formatted -> Formatted
-addFormatting f = addSuffix . pref . quote . font . text_case
-  where pref i = case prefix f of
+addFormatting f =
+  addLink . addSuffix . pref . quote . font . text_case . strip_periods
+  where addLink i = case hyperlink f of
+                         ""  -> i
+                         url -> Formatted [Link nullAttr (unFormatted i) (url, "")]
+        pref i = case prefix f of
                       "" -> i
                       x  -> fromString x <> i
         addSuffix i
@@ -144,6 +145,10 @@ addFormatting f = addSuffix . pref . quote . font . text_case
                                   = i <> fromString (tail $ suffix f)
           | otherwise             = i <> fromString (suffix f)
 
+        strip_periods (Formatted ils) = Formatted (walk removePeriod ils)
+        removePeriod (Str xs) | stripPeriods f = Str (filter (/='.') xs)
+        removePeriod x        = x
+
         quote (Formatted [])  = Formatted []
         quote (Formatted ils) =
                     case quotes f of
