@@ -1,4 +1,5 @@
 {-# OPTIONS_GHC -fno-warn-unused-do-bind #-}
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Input.Bibtex
@@ -49,14 +50,21 @@ adjustSpans _ x = [x]
 
 parseRawLaTeX :: Lang -> String -> [Inline]
 parseRawLaTeX lang ('\\':xs) =
-  case readLaTeX def{readerParseRaw = True} contents of
-       Pandoc _ [Para ys]  -> f command ys
-       Pandoc _ [Plain ys] -> f command ys
-       _                   -> []
+  case latex' contents of
+       [Para ys]  -> f command ys
+       [Plain ys] -> f command ys
+       _          -> []
    where (command', contents') = break (=='{') xs
          command  = trim command'
          contents = drop 1 $ reverse $ drop 1 $ reverse contents'
-         f "mkbibquote" ils = [Quoted DoubleQuote ils]
+         f "mkbibquote"    ils = [Quoted DoubleQuote ils]
+         f "mkbibemph"     ils = [Emph ils]
+         f "mkbibitalic"   ils = [Emph ils] -- TODO: italic/=emph
+         f "mkbibbold"     ils = [Strong ils]
+         f "mkbibparens"   ils = [Str "("] ++ ils ++ [Str ")"] -- TODO: ...
+         f "mkbibbrackets" ils = [Str "["] ++ ils ++ [Str "]"] -- TODO: ...
+         -- ... both should be nestable & should work in year fields
+         f "autocap"    ils = ils  -- TODO: should work in year fields
          f "textnormal" ils = [Span ("",["nodecor"],[]) ils]
          f "bibstring" [Str s] = [Str $ resolveKey' lang s]
          f _            ils = [Span nullAttr ils]
@@ -79,10 +87,10 @@ readBibtexInputString :: Bool -> String -> IO [Reference]
 readBibtexInputString isBibtex bibstring = do
   env <- getEnvironment
   let lang = case lookup "LANG" env of
-                  Just x  -> case splitWhen (\c -> c == '.' || c == '_') x of
-                                   (w:z:_) -> Lang w z
-                                   [w]     -> Lang w mempty
-                                   _       -> Lang "en" "US"
+                  Just x  -> case splitWhen (\c -> c == '.' || c == '_' || c == '-') x of
+                                   (w:z:_)            -> Lang w z
+                                   [w] | not (null w) -> Lang w mempty
+                                   _                  -> Lang "en" "US"
                   Nothing -> Lang "en" "US"
   let items = case runParser (bibEntries <* eof) [] "stdin" bibstring of
                    Left err -> error (show err)
@@ -228,8 +236,8 @@ getXrefFields isBibtex baseEntry entries keys = do
        return (k',v)
 
 resolveCrossRef :: Bool -> [Item] -> Item -> Item
-resolveCrossRef isBibtex entries entry = foldl go entry (fields entry)
-  where go entry' (key, val) =
+resolveCrossRef isBibtex entries entry = foldr go entry (fields entry)
+  where go (key, val) entry' =
           if key == "crossref" || key == "xdata"
           then entry'{ fields = fields entry' ++
                                     getXrefFields isBibtex entry entries val }
@@ -316,47 +324,494 @@ data Lang = Lang String String  -- e.g. "en" "US"
 langToLocale :: Lang -> String
 langToLocale (Lang x y) = x ++ ('-':y)
 
+-- Biblatex Localization Keys (see Biblatex manual)
+-- Currently we only map a subset likely to be used in Biblatex *databases*
+-- (in fields such as `type`, and via `\bibstring{}` commands).
+
 resolveKey :: Lang -> Formatted -> Formatted
 resolveKey lang (Formatted ils) = Formatted (Walk.walk go ils)
   where go (Str s) = Str $ resolveKey' lang s
         go x       = x
 
+-- biblatex localization keys, from files at
+-- http://github.com/plk/biblatex/tree/master/tex/latex/biblatex/lbx
+-- Some keys missing in these were added from csl locale files at
+-- http://github.com/citation-style-language/locales -- labeled "csl"
 resolveKey' :: Lang -> String -> String
+resolveKey' (Lang "ca" "AD") k =
+    case map toLower k of
+       "inpreparation" -> "en preparació"
+       "submitted"     -> "enviat"
+       "forthcoming"   -> "disponible en breu"
+       "inpress"       -> "a impremta"
+       "prepublished"  -> "pre-publicat"
+       "mathesis"      -> "tesi de màster"
+       "phdthesis"     -> "tesi doctoral"
+       "candthesis"    -> "tesi de candidatura"
+       "techreport"    -> "informe tècnic"
+       "resreport"     -> "informe de recerca"
+       "software"      -> "programari"
+       "datacd"        -> "CD de dades"
+       "audiocd"       -> "CD d’àudio"
+       "patent"        -> "patent"
+       "patentde"      -> "patent alemana"
+       "patenteu"      -> "patent europea"
+       "patentfr"      -> "patent francesa"
+       "patentuk"      -> "patent britànica"
+       "patentus"      -> "patent estatunidenca"
+       "patreq"        -> "soŀlicitud de patent"
+       "patreqde"      -> "soŀlicitud de patent alemana"
+       "patreqeu"      -> "soŀlicitud de patent europea"
+       "patreqfr"      -> "soŀlicitud de patent francesa"
+       "patrequk"      -> "soŀlicitud de patent britànica"
+       "patrequs"      -> "soŀlicitud de patent estatunidenca"
+       "countryde"     -> "Alemanya"
+       "countryeu"     -> "Unió Europea"
+       "countryep"     -> "Unió Europea"
+       "countryfr"     -> "França"
+       "countryuk"     -> "Regne Unit"
+       "countryus"     -> "Estats Units d’Amèrica"
+       "newseries"     -> "sèrie nova"
+       "oldseries"     -> "sèrie antiga"
+       _               -> k
+resolveKey' (Lang "da" "DK") k =
+    case map toLower k of
+       -- "inpreparation" -> "" -- missing
+       -- "submitted"     -> "" -- missing
+       "forthcoming"   -> "kommende" -- csl
+       "inpress"       -> "i tryk"   -- csl
+       -- "prepublished"  -> "" -- missing
+       "mathesis"      -> "speciale"
+       "phdthesis"     -> "ph.d.-afhandling"
+       "candthesis"    -> "kandidatafhandling"
+       "techreport"    -> "teknisk rapport"
+       "resreport"     -> "forskningsrapport"
+       "software"      -> "software"
+       "datacd"        -> "data-cd"
+       "audiocd"       -> "lyd-cd"
+       "patent"        -> "patent"
+       "patentde"      -> "tysk patent"
+       "patenteu"      -> "europæisk patent"
+       "patentfr"      -> "fransk patent"
+       "patentuk"      -> "britisk patent"
+       "patentus"      -> "amerikansk patent"
+       "patreq"        -> "ansøgning om patent"
+       "patreqde"      -> "ansøgning om tysk patent"
+       "patreqeu"      -> "ansøgning om europæisk patent"
+       "patreqfr"      -> "ansøgning om fransk patent"
+       "patrequk"      -> "ansøgning om britisk patent"
+       "patrequs"      -> "ansøgning om amerikansk patent"
+       "countryde"     -> "Tyskland"
+       "countryeu"     -> "Europæiske Union"
+       "countryep"     -> "Europæiske Union"
+       "countryfr"     -> "Frankrig"
+       "countryuk"     -> "Storbritanien"
+       "countryus"     -> "USA"
+       "newseries"     -> "ny serie"
+       "oldseries"     -> "gammel serie"
+       _               -> k
+resolveKey' (Lang "de" "DE") k =
+    case map toLower k of
+       "inpreparation" -> "in Vorbereitung"
+       "submitted"     -> "eingereicht"
+       "forthcoming"   -> "im Erscheinen"
+       "inpress"       -> "im Druck"
+       "prepublished"  -> "Vorveröffentlichung"
+       "mathesis"      -> "Magisterarbeit"
+       "phdthesis"     -> "Dissertation"
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "Technischer Bericht"
+       "resreport"     -> "Forschungsbericht"
+       "software"      -> "Computer-Software"
+       "datacd"        -> "CD-ROM"
+       "audiocd"       -> "Audio-CD"
+       "patent"        -> "Patent"
+       "patentde"      -> "deutsches Patent"
+       "patenteu"      -> "europäisches Patent"
+       "patentfr"      -> "französisches Patent"
+       "patentuk"      -> "britisches Patent"
+       "patentus"      -> "US-Patent"
+       "patreq"        -> "Patentanmeldung"
+       "patreqde"      -> "deutsche Patentanmeldung"
+       "patreqeu"      -> "europäische Patentanmeldung"
+       "patreqfr"      -> "französische Patentanmeldung"
+       "patrequk"      -> "britische Patentanmeldung"
+       "patrequs"      -> "US-Patentanmeldung"
+       "countryde"     -> "Deutschland"
+       "countryeu"     -> "Europäische Union"
+       "countryep"     -> "Europäische Union"
+       "countryfr"     -> "Frankreich"
+       "countryuk"     -> "Großbritannien"
+       "countryus"     -> "USA"
+       "newseries"     -> "neue Folge"
+       "oldseries"     -> "alte Folge"
+       _               -> k
 resolveKey' (Lang "en" "US") k =
   case map toLower k of
-       "inpreparation" -> "in preparation"
-       "submitted"     -> "submitted"
-       "forthcoming"   -> "forthcoming"
-       "inpress"       -> "in press"
-       "prepublished"  -> "pre-published"
-       "mathesis"      -> "Master’s thesis"
-       "phdthesis"     -> "PhD thesis"
-       "candthesis"    -> "Candidate thesis"
-       "techreport"    -> "technical report"
-       "resreport"     -> "research report"
-       "software"      -> "computer software"
-       "datacd"        -> "data CD"
+       "audiocd"        -> "audio CD"
+       "by"             -> "by"
+       "candthesis"     -> "Candidate thesis"
+       "countryde"      -> "Germany"
+       "countryep"      -> "European Union"
+       "countryeu"      -> "European Union"
+       "countryfr"      -> "France"
+       "countryuk"      -> "United Kingdom"
+       "countryus"      -> "United States of America"
+       "datacd"         -> "data CD"
+       "edition"        -> "ed."
+       "forthcoming"    -> "forthcoming"
+       "inpreparation"  -> "in preparation"
+       "inpress"        -> "in press"
+       "introduction"   -> "introduction"
+       "jourser"        -> "ser."
+       "mathesis"       -> "Master’s thesis"
+       "newseries"      -> "new series"
+       "nodate"         -> "n. d."
+       "number"         -> "no."
+       "numbers"        -> "nos."
+       "oldseries"      -> "old series"
+       "patent"         -> "patent"
+       "patentde"       -> "German patent"
+       "patenteu"       -> "European patent"
+       "patentfr"       -> "French patent"
+       "patentuk"       -> "British patent"
+       "patentus"       -> "U.S. patent"
+       "patreq"         -> "patent request"
+       "patreqde"       -> "German patent request"
+       "patreqeu"       -> "European patent request"
+       "patreqfr"       -> "French patent request"
+       "patrequk"       -> "British patent request"
+       "patrequs"       -> "U.S. patent request"
+       "phdthesis"      -> "PhD thesis"
+       "prepublished"   -> "pre-published"
+       "pseudonym"      -> "pseud."
+       "recorded"       -> "recorded"
+       "resreport"      -> "research report"
+       "reviewof"       -> "Review of"
+       "revisededition" -> "rev. ed."
+       "software"       -> "computer software"
+       "submitted"      -> "submitted"
+       "techreport"     -> "technical report"
+       "volume"         -> "vol."
+       _               -> k
+resolveKey' (Lang "es" "ES") k =
+    case map toLower k of
+       -- "inpreparation" -> "" -- missing
+       -- "submitted"     -> "" -- missing
+       "forthcoming"   -> "previsto"    -- csl
+       "inpress"       -> "en imprenta" -- csl
+       -- "prepublished"  -> "" -- missing
+       "mathesis"      -> "Tesis de licenciatura"
+       "phdthesis"     -> "Tesis doctoral"
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "informe técnico"
+       -- "resreport"  -> "" -- missing
+       -- "software"   -> "" -- missing
+       -- "datacd"     -> "" -- missing
+       -- "audiocd"    -> "" -- missing
+       "patent"        -> "patente"
+       "patentde"      -> "patente alemana"
+       "patenteu"      -> "patente europea"
+       "patentfr"      -> "patente francesa"
+       "patentuk"      -> "patente británica"
+       "patentus"      -> "patente americana"
+       "patreq"        -> "solicitud de patente"
+       "patreqde"      -> "solicitud de patente alemana"
+       "patreqeu"      -> "solicitud de patente europea"
+       "patreqfr"      -> "solicitud de patente francesa"
+       "patrequk"      -> "solicitud de patente británica"
+       "patrequs"      -> "solicitud de patente americana"
+       "countryde"     -> "Alemania"
+       "countryeu"     -> "Unión Europea"
+       "countryep"     -> "Unión Europea"
+       "countryfr"     -> "Francia"
+       "countryuk"     -> "Reino Unido"
+       "countryus"     -> "Estados Unidos de América"
+       "newseries"     -> "nueva época"
+       "oldseries"     -> "antigua época"
+       _               -> k
+resolveKey' (Lang "fi" "FI") k =
+    case map toLower k of
+       -- "inpreparation" -> ""      -- missing
+       -- "submitted"     -> ""      -- missing
+       "forthcoming"   -> "tulossa"  -- csl
+       "inpress"       -> "painossa" -- csl
+       -- "prepublished"  -> ""      -- missing
+       "mathesis"      -> "tutkielma"
+       "phdthesis"     -> "tohtorinväitöskirja"
+       "candthesis"    -> "kandidat"
+       "techreport"    -> "tekninen raportti"
+       "resreport"     -> "tutkimusraportti"
+       "software"      -> "ohjelmisto"
+       "datacd"        -> "data-CD"
+       "audiocd"       -> "ääni-CD"
+       "patent"        -> "patentti"
+       "patentde"      -> "saksalainen patentti"
+       "patenteu"      -> "Euroopan Unionin patentti"
+       "patentfr"      -> "ranskalainen patentti"
+       "patentuk"      -> "englantilainen patentti"
+       "patentus"      -> "yhdysvaltalainen patentti"
+       "patreq"        -> "patenttihakemus"
+       "patreqde"      -> "saksalainen patenttihakemus"
+       "patreqeu"      -> "Euroopan Unionin patenttihakemus"
+       "patreqfr"      -> "ranskalainen patenttihakemus"
+       "patrequk"      -> "englantilainen patenttihakemus"
+       "patrequs"      -> "yhdysvaltalainen patenttihakemus"
+       "countryde"     -> "Saksa"
+       "countryeu"     -> "Euroopan Unioni"
+       "countryep"     -> "Euroopan Unioni"
+       "countryfr"     -> "Ranska"
+       "countryuk"     -> "Iso-Britannia"
+       "countryus"     -> "Yhdysvallat"
+       "newseries"     -> "uusi sarja"
+       "oldseries"     -> "vanha sarja"
+       _               -> k
+
+resolveKey' (Lang "fr" "FR") k =
+    case map toLower k of
+       "inpreparation" -> "en préparation"
+       "submitted"     -> "soumis"
+       "forthcoming"   -> "à paraître"
+       "inpress"       -> "sous presse"
+       "prepublished"  -> "prépublié"
+       "mathesis"      -> "mémoire de master"
+       "phdthesis"     -> "thèse de doctorat"
+       "candthesis"    -> "thèse de candidature"
+       "techreport"    -> "rapport technique"
+       "resreport"     -> "rapport scientifique"
+       "software"      -> "logiciel"
+       "datacd"        -> "cédérom"
+       "audiocd"       -> "disque compact audio"
+       "patent"        -> "brevet"
+       "patentde"      -> "brevet allemand"
+       "patenteu"      -> "brevet européen"
+       "patentfr"      -> "brevet français"
+       "patentuk"      -> "brevet britannique"
+       "patentus"      -> "brevet américain"
+       "patreq"        -> "demande de brevet"
+       "patreqde"      -> "demande de brevet allemand"
+       "patreqeu"      -> "demande de brevet européen"
+       "patreqfr"      -> "demande de brevet français"
+       "patrequk"      -> "demande de brevet britannique"
+       "patrequs"      -> "demande de brevet américain"
+       "countryde"     -> "Allemagne"
+       "countryeu"     -> "Union européenne"
+       "countryep"     -> "Union européenne"
+       "countryfr"     -> "France"
+       "countryuk"     -> "Royaume-Uni"
+       "countryus"     -> "États-Unis"
+       "newseries"     -> "nouvelle série"
+       "oldseries"     -> "ancienne série"
+       _               -> k
+resolveKey' (Lang "it" "IT") k =
+    case map toLower k of
+       -- "inpreparation" -> "" -- missing
+       -- "submitted"     -> "" -- missing
+       "forthcoming"   -> "futuro" -- csl
+       "inpress"       -> "in stampa"
+       -- "prepublished"  -> "" -- missing
+       "mathesis"      -> "tesi di laurea magistrale"
+       "phdthesis"     -> "tesi di dottorato"
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "rapporto tecnico"
+       "resreport"     -> "rapporto di ricerca"
+       -- "software"   -> "" -- missing
+       -- "datacd"     -> "" -- missing
+       -- "audiocd"    -> "" -- missing
+       "patent"        -> "brevetto"
+       "patentde"      -> "brevetto tedesco"
+       "patenteu"      -> "brevetto europeo"
+       "patentfr"      -> "brevetto francese"
+       "patentuk"      -> "brevetto britannico"
+       "patentus"      -> "brevetto americano"
+       "patreq"        -> "brevetto richiesto"
+       "patreqde"      -> "brevetto tedesco richiesto"
+       "patreqeu"      -> "brevetto europeo richiesto"
+       "patreqfr"      -> "brevetto francese richiesto"
+       "patrequk"      -> "brevetto britannico richiesto"
+       "patrequs"      -> "brevetto U.S.A. richiesto"
+       "countryde"     -> "Germania"
+       "countryeu"     -> "Unione Europea"
+       "countryep"     -> "Unione Europea"
+       "countryfr"     -> "Francia"
+       "countryuk"     -> "Regno Unito"
+       "countryus"     -> "Stati Uniti d’America"
+       "newseries"     -> "nuova serie"
+       "oldseries"     -> "vecchia serie"
+       _               -> k
+resolveKey' (Lang "nl" "NL") k =
+    case map toLower k of
+       "inpreparation" -> "in voorbereiding"
+       "submitted"     -> "ingediend"
+       "forthcoming"   -> "onderweg"
+       "inpress"       -> "in druk"
+       "prepublished"  -> "voorpublicatie"
+       "mathesis"      -> "masterscriptie"
+       "phdthesis"     -> "proefschrift"
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "technisch rapport"
+       "resreport"     -> "onderzoeksrapport"
+       "software"      -> "computersoftware"
+       "datacd"        -> "cd-rom"
+       "audiocd"       -> "audio-cd"
+       "patent"        -> "patent"
+       "patentde"      -> "Duits patent"
+       "patenteu"      -> "Europees patent"
+       "patentfr"      -> "Frans patent"
+       "patentuk"      -> "Brits patent"
+       "patentus"      -> "Amerikaans patent"
+       "patreq"        -> "patentaanvraag"
+       "patreqde"      -> "Duitse patentaanvraag"
+       "patreqeu"      -> "Europese patentaanvraag"
+       "patreqfr"      -> "Franse patentaanvraag"
+       "patrequk"      -> "Britse patentaanvraag"
+       "patrequs"      -> "Amerikaanse patentaanvraag"
+       "countryde"     -> "Duitsland"
+       "countryeu"     -> "Europese Unie"
+       "countryep"     -> "Europese Unie"
+       "countryfr"     -> "Frankrijk"
+       "countryuk"     -> "Verenigd Koninkrijk"
+       "countryus"     -> "Verenigde Staten van Amerika"
+       "newseries"     -> "nieuwe reeks"
+       "oldseries"     -> "oude reeks"
+       _               -> k
+resolveKey' (Lang "pl" "PL") k =
+    case map toLower k of
+       "inpreparation" -> "przygotowanie"
+       "submitted"     -> "prezentacja"
+       "forthcoming"   -> "przygotowanie"
+       "inpress"       -> "wydrukowane"
+       "prepublished"  -> "przedwydanie"
+       "mathesis"      -> "praca magisterska"
+       "phdthesis"     -> "praca doktorska"
+       "techreport"    -> "sprawozdanie techniczne"
+       "resreport"     -> "sprawozdanie naukowe"
+       "software"      -> "oprogramowanie"
+       "datacd"        -> "CD-ROM"
        "audiocd"       -> "audio CD"
        "patent"        -> "patent"
-       "patentde"      -> "German patent"
-       "patenteu"      -> "European patent"
-       "patentfr"      -> "French patent"
-       "patentuk"      -> "British patent"
-       "patentus"      -> "U.S. patent"
-       "patreq"        -> "patent request"
-       "patreqde"      -> "German patent request"
-       "patreqeu"      -> "European patent request"
-       "patreqfr"      -> "French patent request"
-       "patrequk"      -> "British patent request"
-       "patrequs"      -> "U.S. patent request"
-       "countryde"     -> "Germany"
-       "countryeu"     -> "European Union"
-       "countryep"     -> "European Union"
-       "countryfr"     -> "France"
-       "countryuk"     -> "United Kingdom"
-       "countryus"     -> "United States of America"
-       "newseries"     -> "new series"
-       "oldseries"     -> "old series"
+       "patentde"      -> "patent Niemiec"
+       "patenteu"      -> "patent Europy"
+       "patentfr"      -> "patent Francji"
+       "patentuk"      -> "patent Wielkiej Brytanji"
+       "patentus"      -> "patent USA"
+       "patreq"        -> "podanie na patent"
+       "patreqeu"      -> "podanie na patent Europy"
+       "patrequs"      -> "podanie na patent USA"
+       "countryde"     -> "Niemcy"
+       "countryeu"     -> "Unia Europejska"
+       "countryep"     -> "Unia Europejska"
+       "countryfr"     -> "Francja"
+       "countryuk"     -> "Wielka Brytania"
+       "countryus"     -> "Stany Zjednoczone Ameryki"
+       "newseries"     -> "nowa serja"
+       "oldseries"     -> "stara serja"
+       _               -> k
+resolveKey' (Lang "pt" "PT") k =
+    case map toLower k of
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "relatório técnico"
+       "resreport"     -> "relatório de pesquisa"
+       "software"      -> "software"
+       "datacd"        -> "CD-ROM"
+       "patent"        -> "patente"
+       "patentde"      -> "patente alemã"
+       "patenteu"      -> "patente européia"
+       "patentfr"      -> "patente francesa"
+       "patentuk"      -> "patente britânica"
+       "patentus"      -> "patente americana"
+       "patreq"        -> "pedido de patente"
+       "patreqde"      -> "pedido de patente alemã"
+       "patreqeu"      -> "pedido de patente européia"
+       "patreqfr"      -> "pedido de patente francesa"
+       "patrequk"      -> "pedido de patente britânica"
+       "patrequs"      -> "pedido de patente americana"
+       "countryde"     -> "Alemanha"
+       "countryeu"     -> "União Europeia"
+       "countryep"     -> "União Europeia"
+       "countryfr"     -> "França"
+       "countryuk"     -> "Reino Unido"
+       "countryus"     -> "Estados Unidos da América"
+       "newseries"     -> "nova série"
+       "oldseries"     -> "série antiga"
+       -- "inpreparation" -> "" -- missing
+       "forthcoming"   -> "a publicar" -- csl
+       "inpress"       -> "na imprensa"
+       -- "prepublished"  -> "" -- missing
+       "mathesis"      -> "tese de mestrado"
+       "phdthesis"     -> "tese de doutoramento"
+       "audiocd"       -> "CD áudio"
+       _               -> k
+resolveKey' (Lang "pt" "BR") k =
+    case map toLower k of       
+       -- "candthesis" -> "" -- missing
+       "techreport"    -> "relatório técnico"
+       "resreport"     -> "relatório de pesquisa"
+       "software"      -> "software"
+       "datacd"        -> "CD-ROM"
+       "patent"        -> "patente"
+       "patentde"      -> "patente alemã"
+       "patenteu"      -> "patente européia"
+       "patentfr"      -> "patente francesa"
+       "patentuk"      -> "patente britânica"
+       "patentus"      -> "patente americana"
+       "patreq"        -> "pedido de patente"
+       "patreqde"      -> "pedido de patente alemã"
+       "patreqeu"      -> "pedido de patente européia"
+       "patreqfr"      -> "pedido de patente francesa"
+       "patrequk"      -> "pedido de patente britânica"
+       "patrequs"      -> "pedido de patente americana"
+       "countryde"     -> "Alemanha"
+       "countryeu"     -> "União Europeia"
+       "countryep"     -> "União Europeia"
+       "countryfr"     -> "França"
+       "countryuk"     -> "Reino Unido"
+       "countryus"     -> "Estados Unidos da América"
+       "newseries"     -> "nova série"
+       "oldseries"     -> "série antiga"
+       "inpreparation" -> "em preparação"
+       "forthcoming"   -> "aceito para publicação"
+       "inpress"       -> "no prelo"
+       "prepublished"  -> "pré-publicado"
+       "mathesis"      -> "dissertação de mestrado"
+       "phdthesis"     -> "tese de doutorado"
+       "audiocd"       -> "CD de áudio"
+       _               -> k
+resolveKey' (Lang "sv" "SE") k =
+    case map toLower k of
+       -- "inpreparation" -> "" -- missing
+       -- "submitted"     -> "" -- missing
+       "forthcoming"   -> "kommande" -- csl
+       "inpress"       -> "i tryck"  -- csl
+       -- "prepublished"  -> "" -- missing
+       "mathesis"      -> "examensarbete"
+       "phdthesis"     -> "doktorsavhandling"
+       "candthesis"    -> "kandidatavhandling"
+       "techreport"    -> "teknisk rapport"
+       "resreport"     -> "forskningsrapport"
+       "software"      -> "datorprogram"
+       "datacd"        -> "data-cd"
+       "audiocd"       -> "ljud-cd"
+       "patent"        -> "patent"
+       "patentde"      -> "tyskt patent"
+       "patenteu"      -> "europeiskt patent"
+       "patentfr"      -> "franskt patent"
+       "patentuk"      -> "brittiskt patent"
+       "patentus"      -> "amerikanskt patent"
+       "patreq"        -> "patentansökan"
+       "patreqde"      -> "ansökan om tyskt patent"
+       "patreqeu"      -> "ansökan om europeiskt patent"
+       "patreqfr"      -> "ansökan om franskt patent"
+       "patrequk"      -> "ansökan om brittiskt patent"
+       "patrequs"      -> "ansökan om amerikanskt patent"
+       "countryde"     -> "Tyskland"
+       "countryeu"     -> "Europeiska unionen"
+       "countryep"     -> "Europeiska unionen"
+       "countryfr"     -> "Frankrike"
+       "countryuk"     -> "Storbritannien"
+       "countryus"     -> "USA"
+       "newseries"     -> "ny följd"
+       "oldseries"     -> "gammal följd"
        _               -> k
 resolveKey' _ k = resolveKey' (Lang "en" "US") k
 
@@ -537,7 +992,7 @@ toAuthor _ [Str "others"] = return $
           , nameSuffix      = mempty
           , literal         = Formatted [Str "others"]
           , commaSuffix     = False
-          , parseNames      = True
+          , parseNames      = False
           }
 toAuthor _ [Span ("",[],[]) ils] =
   return $ -- corporate author
@@ -548,7 +1003,7 @@ toAuthor _ [Span ("",[],[]) ils] =
           , nameSuffix      = mempty
           , literal         = Formatted ils
           , commaSuffix     = False
-          , parseNames      = True
+          , parseNames      = False
           }
 -- First von Last
 -- von Last, First
@@ -601,7 +1056,7 @@ toAuthor opts ils = do
           , nameSuffix      = suffix
           , literal         = mempty
           , commaSuffix     = usecomma
-          , parseNames      = True
+          , parseNames      = False
           }
 
 isCapitalized :: [Inline] -> Bool
@@ -619,8 +1074,19 @@ optionSet key opts = case lookup key opts of
                       _           -> False
 
 latex' :: String -> [Block]
-latex' s = bs
-  where Pandoc _ bs = readLaTeX def{readerParseRaw = True} s
+latex' s = Walk.walk removeSoftBreak $
+#if MIN_VERSION_pandoc(1,14,0)
+  case readLaTeX def{readerParseRaw = True, readerSmart = True} s of
+                Right (Pandoc _ bs) -> bs
+                _                   -> []
+#else
+  case readLaTeX def{readerParseRaw = True, readerSmart = True} s of
+                Pandoc _ bs         -> bs
+#endif
+
+removeSoftBreak :: Inline -> Inline
+removeSoftBreak SoftBreak = Space
+removeSoftBreak x = x
 
 latex :: String -> Bib Formatted
 latex s = blocksToFormatted $ latex' $ trim s
@@ -745,6 +1211,7 @@ itemToReference lang locale bibtex = bib $ do
   isEvent <- (True <$ (getRawField "eventdate"
                      <|> getRawField "eventtitle"
                      <|> getRawField "venue")) <|> return False
+  reftype' <- resolveKey lang <$> getField "type" <|> return mempty
   let (reftype, refgenre) = case et of
        "article"
          | st == "magazine"  -> (ArticleMagazine,mempty)
@@ -757,10 +1224,12 @@ itemToReference lang locale bibtex = bib $ do
        "electronic"      -> (Webpage,mempty)
        "inbook"          -> (Chapter,mempty)
        "incollection"    -> (Chapter,mempty)
-       "inreference "    -> (Chapter,mempty)
+       "inreference"     -> (EntryEncyclopedia,mempty)
        "inproceedings"   -> (PaperConference,mempty)
        "manual"          -> (Book,mempty)
-       "mastersthesis"   -> (Thesis, Formatted [Str $ resolveKey' lang "mathesis"])
+       "mastersthesis"   -> (Thesis, if reftype' == mempty
+                                        then Formatted [Str $ resolveKey' lang "mathesis"]
+                                        else reftype')
        "misc"            -> (NoType,mempty)
        "mvbook"          -> (Book,mempty)
        "mvcollection"    -> (Book,mempty)
@@ -772,7 +1241,9 @@ itemToReference lang locale bibtex = bib $ do
          | st == "magazine"  -> (ArticleMagazine,mempty)
          | st == "newspaper" -> (ArticleNewspaper,mempty)
          | otherwise         -> (ArticleJournal,mempty)
-       "phdthesis"       -> (Thesis, Formatted [Str $ resolveKey' lang "phdthesis"])
+       "phdthesis"       -> (Thesis, if reftype' == mempty
+                                        then Formatted [Str $ resolveKey' lang "phdthesis"]
+                                        else reftype')
        "proceedings"     -> (Book,mempty)
        "reference"       -> (Book,mempty)
        "report"          -> (Report,mempty)
@@ -786,7 +1257,7 @@ itemToReference lang locale bibtex = bib $ do
        "thesis"          -> (Thesis,mempty)
        "unpublished"     -> (if isEvent then Speech else Manuscript,mempty)
        "www"             -> (Webpage,mempty)
-       -- biblatex, "unsupporEd"
+       -- biblatex, "unsupported"
        "artwork"         -> (Graphic,mempty)
        "audio"           -> (Song,mempty)         -- for audio *recordings*
        "commentary"      -> (Book,mempty)
@@ -807,11 +1278,6 @@ itemToReference lang locale bibtex = bib $ do
        "letters"         -> (PersonalCommunication,mempty)
        "newsarticle"     -> (ArticleNewspaper,mempty)
        _                 -> (NoType,mempty)
-  reftype' <- resolveKey lang <$> getField "type" <|> return mempty
-
-  let isContainer = et `elem` ["book","collection","proceedings","reference",
-                     "mvbook","mvcollection","mvproceedings", "mvreference",
-                     "suppbook","suppcollection"]
 
   -- hyphenation:
   let defaultHyphenation = case lang of
@@ -826,7 +1292,7 @@ itemToReference lang locale bibtex = bib $ do
                 ("english","variant=us")         -> return "american"
                 ("english","variant=usmax")      -> return "american"
                 ("english","variant=uk")         -> return "british"
-                ("english","varant=australian")  -> return "australian"
+                ("english","variant=australian") -> return "australian"
                 ("english","variant=newzealand") -> return "newzealand"
                 (x,_)                            -> return x
   hyphenation <- ((toLocale . map toLower) <$>
@@ -846,10 +1312,11 @@ itemToReference lang locale bibtex = bib $ do
   -- FIXME: add same for editora, editorb, editorc
 
   -- titles
-  let isArticle = et `elem` ["article", "periodical", "suppperiodical"]
+  let isArticle = et `elem` ["article", "periodical", "suppperiodical", "review"]
   let isPeriodical = et == "periodical"
-  let hasVolumes = et `elem`
-         ["inbook","incollection","inproceedings","bookinbook"]
+  let isChapterlike = et `elem`
+         ["inbook","incollection","inproceedings","inreference","bookinbook"]
+  hasMaintitle <- (True <$ (getRawField "maintitle")) <|> return False
   let hyphenation' = if null hyphenation
                      then defaultHyphenation
                      else hyphenation
@@ -857,45 +1324,44 @@ itemToReference lang locale bibtex = bib $ do
                       (x:_) -> x
                       []    -> mempty
   modify $ \s -> s{ untitlecase = la == "en" }
-  title' <- getTitle (if isPeriodical then "issuetitle" else "title")
-           <|> return mempty
-  subtitle' <- getTitle (if isPeriodical then "issuesubtitle" else "subtitle")
-              <|> return mempty
-  titleaddon' <- getTitle "titleaddon"
+
+  title' <- (guard isPeriodical >> getTitle "issuetitle")
+            <|> (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "maintitle")
+            <|> getTitle "title"
+            <|> return mempty
+  subtitle' <- (guard isPeriodical >> getTitle "issuesubtitle")
+               <|> (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "mainsubtitle")
+               <|> getTitle "subtitle"
                <|> return mempty
-  volumeTitle' <- (getTitle "maintitle" >> guard hasVolumes
-                    >> getTitle "booktitle")
+  titleaddon' <- (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "maintitleaddon")
+                 <|> getTitle "titleaddon"
+                 <|> return mempty
+  volumeTitle' <- (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "title")
+                  <|> (guard hasMaintitle >> guard isChapterlike >> getTitle "booktitle")
                   <|> return mempty
-  volumeSubtitle' <- (getTitle "maintitle" >> guard hasVolumes
-                      >> getTitle "booksubtitle")
+  volumeSubtitle' <- (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "subtitle")
+                     <|> (guard hasMaintitle >> guard isChapterlike >> getTitle "booksubtitle")
                      <|> return mempty
-  volumeTitleAddon' <- (getTitle "maintitle" >> guard hasVolumes
-                                   >> getTitle "booktitleaddon")
+  volumeTitleAddon' <- (guard hasMaintitle >> guard (not isChapterlike) >> getTitle "titleaddon")
+                       <|> (guard hasMaintitle >> guard isChapterlike >> getTitle "booktitleaddon")
                        <|> return mempty
   containerTitle' <- (guard isPeriodical >> getPeriodicalTitle "title")
-                  <|> getTitle "maintitle"
-                  <|> (guard (not isContainer) >>
-                       guard (volumeTitle' == mempty) >> getTitle "booktitle")
-                  <|> getPeriodicalTitle "journaltitle"
-                  <|> getPeriodicalTitle "journal"
-                  <|> return mempty
+                     <|> (guard isChapterlike >> getTitle "maintitle")
+                     <|> (guard isChapterlike >> getTitle "booktitle")
+                     <|> getPeriodicalTitle "journaltitle"
+                     <|> getPeriodicalTitle "journal"
+                     <|> return mempty
   containerSubtitle' <- (guard isPeriodical >> getPeriodicalTitle "subtitle")
-                       <|> getTitle "mainsubtitle"
-                       <|> (guard (not isContainer) >>
-                            guard (volumeSubtitle' == mempty) >>
-                             getTitle "booksubtitle")
-                       <|> getPeriodicalTitle "journalsubtitle"
-                       <|> return mempty
+                        <|> (guard isChapterlike >> getTitle "mainsubtitle")
+                        <|> (guard isChapterlike >> getTitle "booksubtitle")
+                        <|> getPeriodicalTitle "journalsubtitle"
+                        <|> return mempty
   containerTitleAddon' <- (guard isPeriodical >> getPeriodicalTitle "titleaddon")
-                       <|> getTitle "maintitleaddon"
-                       <|> (guard (not isContainer) >>
-                            guard (volumeTitleAddon' == mempty) >>
-                             getTitle "booktitleaddon")
-                       <|> return mempty
-  containerTitleShort' <- (guard isPeriodical >> getField "shorttitle")
-                        <|> (guard (not isContainer) >>
-                             getTitle "booktitleshort")
-                        <|> getPeriodicalTitle "journaltitleshort"
+                          <|> (guard isChapterlike >> getTitle "maintitleaddon")
+                          <|> (guard isChapterlike >> getTitle "booktitleaddon")
+                          <|> return mempty
+  containerTitleShort' <- (guard isPeriodical >> guard (not hasMaintitle)
+                       >> getField "shorttitle")
                         <|> getPeriodicalTitle "shortjournal"
                         <|> return mempty
   -- change numerical series title to e.g. 'series 3'
@@ -905,8 +1371,10 @@ itemToReference lang locale bibtex = bib $ do
       fixSeriesTitle x = x
   seriesTitle' <- (fixSeriesTitle . resolveKey lang) <$>
                       getTitle "series" <|> return mempty
-  shortTitle' <- getTitle "shorttitle"
-               <|> if subtitle' /= mempty
+  shortTitle' <- (guard (not hasMaintitle || isChapterlike) >>
+                        getTitle "shorttitle")
+               <|> if (subtitle' /= mempty || titleaddon' /= mempty) &&
+                      (not hasMaintitle)
                       then getShortTitle False "title"
                       else getShortTitle True  "title"
                <|> return mempty
@@ -987,6 +1455,8 @@ itemToReference lang locale bibtex = bib $ do
          <|> return mempty
   isbn' <- getRawField "isbn" <|> return mempty
   issn' <- getRawField "issn" <|> return mempty
+  pmid' <- getRawField "pmid" <|> return mempty
+  pmcid' <- getRawField "pmcid" <|> return mempty
   callNumber' <- getRawField "library" <|> return mempty
 
   -- notes
@@ -1093,6 +1563,8 @@ itemToReference lang locale bibtex = bib $ do
          , doi                 = Literal doi'
          , isbn                = Literal isbn'
          , issn                = Literal issn'
+         , pmcid               = Literal pmcid'
+         , pmid                = Literal pmid'
          , language            = Literal hyphenation
          , callNumber          = Literal callNumber'
          }
