@@ -146,8 +146,15 @@ getExt :: String -> String
 getExt = takeExtension . map toLower
 
 readYamlBib :: String -> Either String [Reference]
-readYamlBib s = convertRefs $ lookupMeta "references" meta
-  where  Pandoc meta _ = readMarkdown def{readerStandalone = True} s
+readYamlBib s =
+#if MIN_VERSION_pandoc(1,14,0)
+  case readMarkdown def{readerStandalone = True} s of
+       Right (Pandoc meta _) -> convertRefs (lookupMeta "references" meta)
+       Left e                -> Left (show e)
+#else
+  case readMarkdown def{readerStandalone = True} s of
+       Pandoc meta _ -> convertRefs (lookupMeta "references" meta)
+#endif
 
 convertRefs :: Maybe MetaValue -> Either String [Reference]
 convertRefs Nothing = Right []
