@@ -8,9 +8,10 @@ import Text.Pandoc.Builder (setMeta, deleteMeta, Inlines, cite)
 import Text.Pandoc.Shared (stringify)
 import Text.HTML.TagSoup.Entity (lookupEntity)
 import qualified Data.ByteString.Lazy as L
+import System.SetEnv (setEnv)
 import Control.Applicative ((<|>))
 import Data.Aeson
-import Data.Char ( isDigit, isPunctuation, isSpace )
+import Data.Char ( isDigit, isPunctuation, toLower, isSpace )
 import qualified Data.Map as M
 import Text.CSL.Reference hiding (processCites, Value)
 import Text.CSL.Input.Bibutils (readBiblioFile, convertRefs)
@@ -24,52 +25,101 @@ import Text.CSL.Output.Pandoc ( headInline, tailFirstInlineStr, initInline,
 import Text.CSL.Data (getDefaultCSL)
 import Text.Parsec hiding (State, (<|>))
 import Control.Monad
-import Data.Monoid (mempty)
+import qualified Control.Exception as E
 import Control.Monad.State
 import System.FilePath
 import System.Directory (doesFileExist, getAppUserDataDirectory)
-import Text.CSL.Util (findFile, splitStrWhen)
+import Text.CSL.Util (findFile, splitStrWhen, tr', parseRomanNumeral, trim)
+import System.IO.Error (isDoesNotExistError)
+import Data.Maybe (fromMaybe)
 
 -- | Process a 'Pandoc' document by adding citations formatted
 -- according to a CSL style.  Add a bibliography (if one is called
 -- for) at the end of the document.
 processCites :: Style -> [Reference] -> Pandoc -> Pandoc
 processCites style refs (Pandoc m1 b1) =
-  let m2            = setMeta "nocites-wildcards" (mkNociteWildcards refs m1) m1
-      Pandoc m3 b2  = evalState (walkM setHashes $ Pandoc m2 b1) 1
-      grps          = query getCitation $ Pandoc m3 b2
-      m4            = deleteMeta "nocites-wildcards" m3
+  let metanocites   = lookupMeta "nocite" m1
+      nocites       = mkNociteWildcards refs . query getCitation <$> metanocites
+      Pandoc m2 b2  = evalState (walkM setHashes $ Pandoc (deleteMeta "nocite" m1) b1) 1
+      grps          = query getCitation (Pandoc m2 b2) ++ fromMaybe [] nocites
       locMap        = locatorMap style
-      result        = citeproc procOpts style refs (setNearNote style $
+      result        = citeproc procOpts{ linkCitations = isLinkCitations m2}
+                        style refs (setNearNote style $
                         map (map (toCslCite locMap)) grps)
-      cits_map      = M.fromList $ zip grps (citations result)
-      biblioList    = map (renderPandoc' style) (bibliography result)
-      Pandoc m b3   = bottomUp (mvPunct style) . deNote .
-                        topDown (processCite style cits_map) $ Pandoc m4 b2
-      (bs, lastb)    = case reverse b3 of
-                          (Header lev (id',classes,kvs) ys) : xs ->
-                           (reverse xs, [Header lev (id',classes',kvs) ys])
-                            where classes' = "unnumbered" :
-                                       [c | c <- classes, c /= "unnumbered"]
-                          _                                      -> (b3,  [])
+      cits_map      = tr' "cits_map" $ M.fromList $ zip grps (citations result)
+      biblioList    = map (renderPandoc' style) $ zip (bibliography result) (citationIds result)
+      Pandoc m3 bs  = bottomUp (mvPunct style) . deNote .
+                        topDown (processCite style cits_map) $ Pandoc m2 b2
+      m             = case metanocites of
+                            Nothing -> m3
+                            Just x  -> setMeta "nocite" x m3
   in  Pandoc m $ bottomUp (concatMap removeNocaseSpans)
-               $ bs ++
-                 if lookupMeta "suppress-bibliography" m == Just (MetaBool True)
-                    then []
-                    else [Div ("",["references"],[]) (lastb ++ biblioList)]
+               $ insertRefs m biblioList bs
+
+-- if document contains a Div with id="refs", insert
+-- references as its contents.  Otherwise, insert references
+-- at the end of the document in a Div with id="refs"
+insertRefs :: Meta -> [Block] -> [Block] -> [Block]
+insertRefs meta refs bs =
+  if isRefRemove meta
+     then bs
+     else case runState (walkM go bs) False of
+               (bs', True) -> bs'
+               (_, False)  ->
+                  case reverse bs of
+                        (Header lev (id',classes,kvs) ys) : xs ->
+                          reverse xs ++
+                            [Header lev (id',addUnNumbered classes,kvs) ys,
+                             Div ("refs",["references"],[]) refs]
+                        _   -> bs ++ refHeader ++
+                                [Div ("refs",["references"],[]) refs]
+  where go :: Block -> State Bool Block
+        go (Div attr@("refs",_,_) xs) = do
+          put True
+          -- refHeader isn't used if you have an explicit references div
+          return $ Div attr (xs ++ refs)
+        go x = return x
+        addUnNumbered cs = "unnumbered" : [c | c <- cs, c /= "unnumbered"]
+        refHeader = case refTitle meta of
+                     Just ils ->
+                       [Header 1 ("bibliography", ["unnumbered"], []) ils]
+                     _        -> []
+
+refTitle :: Meta -> Maybe [Inline]
+refTitle meta =
+  case lookupMeta "reference-section-title" meta of
+    Just (MetaString s)           -> Just [Str s]
+    Just (MetaInlines ils)        -> Just ils
+    Just (MetaBlocks [Plain ils]) -> Just ils
+    Just (MetaBlocks [Para ils])  -> Just ils
+    _                             -> Nothing
+
+isRefRemove :: Meta -> Bool
+isRefRemove meta =
+  case lookupMeta "suppress-bibliography" meta of
+    Just (MetaBool True) -> True
+    _                    -> False
+
+isLinkCitations :: Meta -> Bool
+isLinkCitations meta =
+  case lookupMeta "link-citations" meta of
+    Just (MetaBool True)   -> True
+    Just (MetaString s)    -> map toLower s `elem` yesValues
+    Just (MetaInlines ils) -> map toLower (stringify ils) `elem` yesValues
+    _                      -> False
+  where yesValues = ["true", "yes", "on"]
 
 -- if the 'nocite' Meta field contains a citation with id = '*',
 -- create a cite with to all the references.
-mkNociteWildcards :: [Reference] -> Meta -> Inlines
-mkNociteWildcards refs meta =
-  case lookupMeta "nocite" meta of
-       Nothing      -> mempty
-       Just nocites ->
-         case [c | c <- concat (query getCitation nocites)
+mkNociteWildcards :: [Reference] -> [[Citation]] -> [[Citation]]
+mkNociteWildcards refs nocites =
+  map (\citgrp -> expandStar citgrp) nocites
+  where expandStar cs =
+         case [c | c <- cs
                  , citationId c == "*"] of
-              []    -> mempty
-              (_:_) -> cite allcites mempty
-         where allcites = map (\ref -> Citation{
+              [] -> cs
+              _  -> allcites
+        allcites = map (\ref -> Citation{
                                   citationId = unLiteral (refId ref),
                                   citationPrefix = [],
                                   citationSuffix = [],
@@ -88,31 +138,55 @@ removeNocaseSpans x = [x]
 -- field.
 processCites' :: Pandoc -> IO Pandoc
 processCites' (Pandoc meta blocks) = do
-  csldir <- getAppUserDataDirectory "csl"
+  mbcsldir <- E.catch (Just <$> getAppUserDataDirectory "csl") $ \e ->
+                 if isDoesNotExistError e
+                    then return Nothing
+                    else E.throwIO e
+  mbpandocdir <- E.catch (Just <$> getAppUserDataDirectory "pandoc") $ \e ->
+                 if isDoesNotExistError e
+                    then return Nothing
+                    else E.throwIO e
   let inlineRefError s = error $ "Error parsing references: " ++ s
   let inlineRefs = either inlineRefError id
                    $ convertRefs $ lookupMeta "references" meta
-  bibRefs <- getBibRefs $ maybe (MetaList []) id
-                        $ lookupMeta "bibliography" meta
-  let refs = inlineRefs ++ bibRefs
   let cslfile = (lookupMeta "csl" meta <|> lookupMeta "citation-style" meta)
                 >>= toPath
-  let mbLocale = lookupMeta "locale" meta >>= toPath
+  let mbLocale = (lookupMeta "lang" meta `mplus` lookupMeta "locale" meta)
+                   >>= toPath
+  let getDefaultCSL' = case mbcsldir of
+                            Just csldir -> do
+                              let f = csldir </> "chicago-author-date.csl"
+                              exists <- doesFileExist f
+                              if exists
+                                 then L.readFile f
+                                 else getDefaultCSL
+                            Nothing -> getDefaultCSL
   csl <- case cslfile of
                Just f | not (null f) -> readCSLFile mbLocale f
                _ -> do
                  -- get default CSL: look first in ~/.csl, and take
                  -- from distribution if not found
-                 let f = csldir </> "chicago-author-date.csl"
-                 exists <- doesFileExist f
-                 raw <- if exists
-                           then L.readFile f
-                           else getDefaultCSL
+                 raw <- case mbpandocdir of
+                          Just pandocdir -> do
+                            let f = pandocdir </> "default.csl"
+                            exists <- doesFileExist f
+                            if exists
+                               then L.readFile f
+                               else getDefaultCSL'
+                          Nothing -> getDefaultCSL'
                  localizeCSL mbLocale $ parseCSL' raw
+  -- set LANG environment from locale; this affects unicode collation
+  -- if pandoc-citeproc compiled with unicode_collation flag
+  setEnv "LC_ALL" $ case styleLocale csl of
+                        (l:_) -> localeLang l
+                        _     -> "en-US"
+  bibRefs <- getBibRefs $ maybe (MetaList []) id
+                        $ lookupMeta "bibliography" meta
+  let refs = inlineRefs ++ bibRefs
   let cslAbbrevFile = lookupMeta "citation-abbreviations" meta >>= toPath
   let skipLeadingSpace = L.dropWhile (\s -> s == 32 || (s >= 9 && s <= 13))
   abbrevs <- maybe (return (Abbreviations M.empty))
-             (\f -> findFile [".", csldir] f >>=
+             (\f -> findFile (maybe ["."] (\g -> [".", g]) mbcsldir) f >>=
                     maybe (error $ "Could not find " ++ f) return >>=
                L.readFile >>=
                either error return . eitherDecode . skipLeadingSpace)
@@ -157,11 +231,7 @@ decodeEntities (x:xs) = x : decodeEntities xs
 processCite :: Style -> M.Map [Citation] Formatted -> Inline -> Inline
 processCite s cs (Cite t _) =
    case M.lookup t cs of
-        Just (Formatted (x:xs))
-          | isTextualCitation t && not (null xs) ->
-                         Cite t (renderPandoc s (Formatted [x]) ++
-                                 renderPandoc s (Formatted xs))
-          | otherwise -> Cite t (renderPandoc s (Formatted (x:xs)))
+        Just (Formatted (x:xs)) -> Cite t (renderPandoc s (Formatted (x:xs)))
         _             -> Strong [Str "???"]  -- TODO raise error instead?
 processCite _ _ x = x
 
@@ -175,32 +245,38 @@ mvPunctInsideQuote (Quoted qt ils) (Str s) | s `elem` [".", ","] =
   [Quoted qt (init ils ++ (mvPunctInsideQuote (last ils) (Str s)))]
 mvPunctInsideQuote il il' = [il, il']
 
+isSpacy :: Inline -> Bool
+isSpacy Space = True
+isSpacy SoftBreak = True
+isSpacy _ = False
+
 mvPunct :: Style -> [Inline] -> [Inline]
-mvPunct _ (Space : Space : xs) = Space : xs
-mvPunct _ (Space : x : ys) | isNote x, startWithPunct ys =
+mvPunct _ (x : Space : xs) | isSpacy x = x : xs
+mvPunct _ (s : x : ys) | isSpacy s, isNote x, startWithPunct ys =
    Str (headInline ys) : x : tailFirstInlineStr ys
 mvPunct _ (Cite cs ils : ys) |
      length ils > 1
    , isNote (last ils)
    , startWithPunct ys
-   = Cite cs (init ils ++ [Str (headInline ys) | not (endWithPunct (init ils))]
+   = Cite cs (init ils ++ [Str (headInline ys) | not (endWithPunct False (init ils))]
      ++ [last ils]) : tailFirstInlineStr ys
 mvPunct sty (q@(Quoted _ _) : w@(Str _) : x : ys)
   | isNote x, isPunctuationInQuote sty  =
     mvPunctInsideQuote q w ++ (x : ys)
-mvPunct _ (Space : x : ys) | isNote x = x : ys
-mvPunct _ (Space : x@(Cite _ (Superscript _ : _)) : ys) = x : ys
+mvPunct _ (s : x : ys) | isSpacy s, isNote x = x : ys
+mvPunct _ (s : x@(Cite _ (Superscript _ : _)) : ys) | isSpacy s = x : ys
 mvPunct _ xs = xs
 
-endWithPunct :: [Inline] -> Bool
-endWithPunct [] = True
-endWithPunct xs@(_:_) = case reverse (stringify xs) of
-                              []                       -> True
-                              -- covers .), .", etc.:
-                              (d:c:_) | isPunctuation d
-                                       && isEndPunct c -> True
-                              (c:_) | isEndPunct c     -> True
-                                    | otherwise        -> False
+endWithPunct :: Bool -> [Inline] -> Bool
+endWithPunct _ [] = True
+endWithPunct onlyFinal xs@(_:_) =
+  case reverse (stringify xs) of
+       []                       -> True
+       -- covers .), .", etc.:
+       (d:c:_) | isPunctuation d && not onlyFinal
+                && isEndPunct c -> True
+       (c:_) | isEndPunct c     -> True
+             | otherwise        -> False
   where isEndPunct c = c `elem` (".,;:!?" :: String)
 
 startWithPunct :: [Inline] -> Bool
@@ -212,30 +288,26 @@ deNote = topDown go
             Cite (c:cs) [Note $ sanitize xs]
         go (Note xs) = Note $ topDown go' xs
         go x = x
-        go' (x : Cite cs [Note [Para xs]] : ys) | x /= Space =
+        go' (x : Cite cs [Note [Para xs]] : ys) | not (isSpacy x) =
              x : Str "," : Space : comb (\zs -> [Cite cs zs]) xs ys
-        go' (x : Note [Para xs] : ys) | x /= Space =
+        go' (x : Note [Para xs] : ys) | not (isSpacy x) =
              x : Str "," : Space : comb id xs ys
         go' (Cite cs [Note [Para xs]] : ys) = comb (\zs -> [Cite cs zs]) xs ys
         go' (Note [Para xs] : ys) = comb id xs ys
         go' xs = xs
-        removeLeadingPunct (Str [c] : Space : xs)
-          | c == ',' || c == '.' || c == ':' = xs
+        removeLeadingPunct (Str [c] : s : xs)
+          | isSpacy s && (c == ',' || c == '.' || c == ':') = xs
         removeLeadingPunct xs = xs
         comb f xs ys =
-           let xs' = if startWithPunct ys && endWithPunct xs
+           let xs' = if startWithPunct ys && endWithPunct True xs
                         then initInline $ removeLeadingPunct xs
                         else removeLeadingPunct xs
            in f xs' ++ ys
         sanitize :: [Block] -> [Block]
         sanitize [Para xs] =
-           [Para $ toCapital xs ++ if endWithPunct xs then [Space] else []]
+           [Para $ toCapital xs ++ if endWithPunct False xs then [Space] else []]
         sanitize bs = bs
 
-isTextualCitation :: [Citation] -> Bool
-isTextualCitation (c:_) = citationMode c == AuthorInText
-isTextualCitation _     = False
-
 -- | Retrieve all citations from a 'Pandoc' docuument. To be used with
 -- 'query'.
 getCitation :: Inline -> [[Citation]]
@@ -264,18 +336,14 @@ toCslCite locMap c
                          _                   -> s
           isPunct (Str (x:_)) = isPunctuation x
           isPunct _           = False
-          citMode = case citationMode c of
-                      AuthorInText   -> (True, False)
-                      SuppressAuthor -> (False,True )
-                      NormalCitation -> (False,False)
       in   emptyCite { CSL.citeId         = citationId c
                      , CSL.citePrefix     = Formatted $ citationPrefix c
                      , CSL.citeSuffix     = Formatted s'
                      , CSL.citeLabel      = la
                      , CSL.citeLocator    = lo
                      , CSL.citeNoteNumber = show $ citationNoteNum c
-                     , CSL.authorInText   = fst citMode
-                     , CSL.suppressAuthor = snd citMode
+                     , CSL.authorInText   = citationMode c == AuthorInText
+                     , CSL.suppressAuthor = citationMode c == SuppressAuthor
                      , CSL.citeHash       = citationHash c
                      }
 
@@ -299,22 +367,31 @@ pMatch condition = try $ do
   return t
 
 pSpace :: Parsec [Inline] st Inline
-pSpace = pMatch (\t -> t == Space || t == Str "\160")
+pSpace = pMatch (\t -> isSpacy t || t == Str "\160")
 
 pLocator :: LocatorMap -> Parsec [Inline] st (String, String)
 pLocator locMap = try $ do
   optional $ pMatch (== Str ",")
   optional pSpace
-  rawLoc <- many
-     (notFollowedBy pSpace >> notFollowedBy (pWordWithDigits True) >> anyToken)
-  la <- case stringify rawLoc of
-                 ""   -> lookAhead (optional pSpace >> pDigit) >> return "page"
-                 s    -> maybe mzero return $ parseLocator locMap s
+  la <- try (do ts <- many1 (notFollowedBy (pWordWithDigits True) >> anyToken)
+                case M.lookup (trim (stringify ts)) locMap of
+                       Just l  -> return l
+                       Nothing -> mzero)
+      <|> (lookAhead pDigit >> return "page")
   g <- pWordWithDigits True
   gs <- many (pWordWithDigits False)
   let lo = concat (g:gs)
   return (la, lo)
 
+pRoman :: Parsec [Inline] st String
+pRoman = try $ do
+  t <- anyToken
+  case t of
+       Str xs -> case parseRomanNumeral xs of
+                      Nothing -> mzero
+                      Just _  -> return xs
+       _      -> mzero
+
 -- we want to capture:  123, 123A, C22, XVII, 33-44, 22-33; 22-11
 pWordWithDigits :: Bool -> Parsec [Inline] st String
 pWordWithDigits isfirst = try $ do
@@ -322,9 +399,13 @@ pWordWithDigits isfirst = try $ do
               then return ""
               else stringify `fmap` pLocatorPunct
   sp <- option "" (pSpace >> return " ")
-  r <- many1 (notFollowedBy pSpace >> notFollowedBy pLocatorPunct >> anyToken)
-  let s = stringify r
-  guard $ any isDigit s || all (`elem` ("IVXLCM" :: String)) s
+  s <-  pRoman <|>
+        try (do ts <- many1 (notFollowedBy pSpace >>
+                             notFollowedBy pLocatorPunct >>
+                             anyToken)
+                let ts' = stringify ts
+                guard (any isDigit ts')
+                return ts')
   return $ punct ++ sp ++ s
 
 pDigit :: Parsec [Inline] st ()
@@ -343,9 +424,6 @@ isLocatorPunct _         = False
 
 type LocatorMap = M.Map String String
 
-parseLocator :: LocatorMap -> String -> Maybe String
-parseLocator locMap s = M.lookup s locMap
-
 locatorMap :: Style -> LocatorMap
 locatorMap sty =
   foldr (\term -> M.insert (termSingular term) (cslTerm term)
