@@ -25,7 +25,6 @@ import Data.Text (Text, unpack)
 import Text.CSL.Style hiding (parseNames)
 import Text.CSL.Util (toRead, findFile)
 import System.Directory (getAppUserDataDirectory)
-import Control.Applicative hiding (many, Const)
 import qualified Text.XML as X
 import Data.Default
 import Text.Pandoc.Shared (safeRead)
@@ -220,14 +219,17 @@ parseDate cur = [Date (words variable) form format delim parts partsAttr]
                            _         -> NoFormDate
         format     = getFormatting cur
         delim      = stringAttr "delimiter" cur
-        parts      = cur $/ get "date-part" &| parseDatePart
+        parts      = cur $/ get "date-part" &| (parseDatePart form)
         partsAttr  = stringAttr "date-parts" cur
 
-parseDatePart :: Cursor -> DatePart
-parseDatePart cur =
+parseDatePart :: DateForm -> Cursor -> DatePart
+parseDatePart defaultForm cur =
   DatePart { dpName       = stringAttr "name" cur
            , dpForm       = case stringAttr "form" cur of
-                                  ""  -> "long"
+                                  ""  -> case defaultForm of
+                                              TextDate -> "long"
+                                              NumericDate -> "numeric"
+                                              _ -> "long"
                                   x    -> x
            , dpRangeDelim = case stringAttr "range-delimiter" cur of
                                   ""  -> "-"
