@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, DeriveDataTypeable, OverloadedStrings #-}
+{-# LANGUAGE PatternGuards, OverloadedStrings #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Proc
@@ -22,19 +22,19 @@ import Data.List
 import Data.Ord  ( comparing )
 import Data.Maybe ( mapMaybe )
 import Text.CSL.Eval
-import Text.CSL.Util ( capitalize, proc, proc', query, toShow )
+import Text.CSL.Util ( proc, proc', query, uncamelize, tr' )
 import Text.CSL.Proc.Collapse
 import Text.CSL.Proc.Disamb
 import Text.CSL.Reference
 import Text.CSL.Style
 import Data.Aeson
-import Data.Monoid ((<>))
 import Control.Applicative ((<|>))
 import Text.Pandoc.Definition (Inline(Space, Str, Note), Block(Para))
 
 data ProcOpts
     = ProcOpts
       { bibOpts :: BibOpts
+      , linkCitations :: Bool
       }
     deriving ( Show, Read, Eq )
 
@@ -69,7 +69,10 @@ instance FromJSON BibOpts where
   parseJSON _ = return $ Select [] []
 
 procOpts :: ProcOpts
-procOpts = ProcOpts (Select [] [])
+procOpts = ProcOpts
+      { bibOpts = Select [] []
+      , linkCitations = False
+      }
 
 -- | With a 'Style', a list of 'Reference's and the list of citation
 -- groups (the list of citations with their locator), produce the
@@ -89,21 +92,38 @@ processBibliography ops s rs
 -- and the bibliography.
 citeproc :: ProcOpts -> Style -> [Reference] -> Citations -> BiblioData
 citeproc ops s rs cs
-    = BD citsOutput biblioOutput
+    = BD citsOutput biblioOutput $ map (unLiteral . refId) biblioRefs
     where
       -- the list of bib entries, as a list of Reference, with
       -- position, locator and year suffix set.
       biblioRefs   = procRefs s . mapMaybe (getReference rs) .
                      nubBy (\a b -> citeId a == citeId b) . concat $ cs
       biblioOutput = if "disambiguate-add-year-suffix" `elem` getCitDisambOptions s
-                     then map formatOutputList $
-                          map (proc (updateYearSuffixes yearS) . map addYearSuffix) $
+                     then map (formatOutputList .
+                               proc (updateYearSuffixes yearS) . map addYearSuffix) $
                           procBiblio (bibOpts ops) s biblioRefs
                      else map formatOutputList $
+                          tr' "citeproc:after procBiblio" $
                           procBiblio (bibOpts ops) s biblioRefs
       citsAndRefs  = processCites biblioRefs cs
       (yearS,citG) = disambCitations s biblioRefs cs $ map (procGroup s) citsAndRefs
-      citsOutput   = map (formatCitLayout s) . collapseCitGroups s $ citG
+      citsOutput   = map (formatCitLayout s) .
+                     tr' "citeproc:collapsed" .
+                     collapseCitGroups s .
+                     (if linkCitations ops && styleClass s == "in-text"
+                         then proc addLink
+                         else id) .
+                     tr' "citeproc:citG" $
+                     citG
+      addLink :: (Cite, Output) -> (Cite, Output)
+      addLink (cit, outp) = (cit, proc (addLink' (citeId cit)) outp)
+      addLink' citeid (OYear y _ f) =
+         OYear y citeid f{hyperlink = "#ref-" ++ citeid}
+      addLink' citeid (OYearSuf y _ d f) =
+         OYearSuf y citeid d f{hyperlink = "#ref-" ++ citeid}
+      addLink' citeid (OCitNum n f) =
+         OCitNum n f{hyperlink = "#ref-" ++ citeid}
+      addLink' _ x = x
 
 -- | Given the CSL 'Style' and the list of 'Reference's sort the list
 -- according to the 'Style' and assign the citation number to each
@@ -115,14 +135,14 @@ procRefs (Style {biblio = mb, csMacros = ms , styleLocale = l, styleAbbrevs = as
       opts'   b = mergeOptions (bibOptions b) opts
       setCNum   = map (\(x,y) -> x { citationNumber = fromIntegral y }) . flip zip ([1..] :: [Int])
       sort_   b = evalSorting (EvalSorting emptyCite {citePosition = "first"}) l ms (opts' b) (bibSort b) as
-      process b = setCNum . sortItems . map (id &&& sort_ b) $ rs
+      process b = setCNum . sortItems . map (id &&& sort_ b . Just) $ rs
 
 sortItems :: Show a => [(a,[Sorting])] -> [a]
 sortItems [] = []
 sortItems l
     = case head . concatMap (map snd) $ result of
         [] -> concatMap (map fst) result
-        _  -> if or $ map ((<) 1 . length) result
+        _  -> if any ((<) 1 . length) result
               then concatMap sortItems result
               else concatMap (map fst) result
     where
@@ -136,8 +156,19 @@ sortItems l
 procBiblio :: BibOpts -> Style -> [Reference] -> [[Output]]
 procBiblio bos (Style {biblio = mb, csMacros = ms , styleLocale = l,
                        styleAbbrevs = as, csOptions = opts}) rs
-    = maybe [] process mb
+    = map addSpaceAfterCitNum $ maybe [] process mb
     where
+      -- handle second-field-align (sort of)
+      addSpaceAfterCitNum [Output (OCitNum n f : xs) f']
+        | secondFieldAlign == Just "flush"  =
+            [Output (OCitNum n f : OSpace : xs) f']
+        | secondFieldAlign == Just "margin" =
+            [Output (OCitNum n f : OSpace : xs) f']
+        | otherwise = [Output (OCitNum n f : xs) f']
+      addSpaceAfterCitNum xs = xs
+
+      secondFieldAlign = lookup "second-field-align" $ maybe [] bibOptions mb
+
       process :: Bibliography -> [[Output]]
       process b   = map (formatBiblioLayout (layFormat $ bibLayout b) (layDelim $ bibLayout b)) $ render b
 
@@ -145,8 +176,7 @@ procBiblio bos (Style {biblio = mb, csMacros = ms , styleLocale = l,
       render  b   = subsequentAuthorSubstitute b . map (evalBib b) . filterRefs bos $ rs
 
       evalBib :: Bibliography -> Reference -> [Output]
-      evalBib b r = evalLayout (bibLayout b) (EvalBiblio emptyCite {citePosition = "first"}) False l ms
-                               (mergeOptions (bibOptions b) opts) as r
+      evalBib b = evalLayout (bibLayout b) (EvalBiblio emptyCite {citePosition = "first"}) False l ms (mergeOptions (bibOptions b) opts) as . Just
 
 subsequentAuthorSubstitute :: Bibliography -> [[Output]] -> [[Output]]
 subsequentAuthorSubstitute b = if null subAuthStr then id else chkCreator
@@ -162,8 +192,8 @@ subsequentAuthorSubstitute b = if null subAuthStr then id else chkCreator
                           "partial-each"  ->          query namesQ  . queryContrib
                           _               ->                          queryContrib
 
-      getPartialEach x xs = concat . take 1 . map fst . reverse .
-                            sortBy (comparing $ length . snd) . filter ((<) 0 . length . snd) .
+      getPartialEach x xs = concat . take 1 . map fst .
+                            sortBy (flip (comparing $ length . snd)) . filter ((<) 0 . length . snd) .
                             zip xs . map (takeWhile id . map (uncurry (==)) . zip x) $ xs
 
       chkCreator = if subAuthRule == "partial-each" then chPartialEach [] else chkCr []
@@ -242,25 +272,25 @@ filterRefs bos refs
                         "categories"   -> look "categories"
                         x              -> look x
           where
-            look s = case lookup s (mkRefMap r) of
-                       Just x | Just v' <- (fromValue x :: Maybe RefType  ) -> v == toShow (show v')
+            look s = case lookup s (mkRefMap (Just r)) of
+                       Just x | Just v' <- (fromValue x :: Maybe RefType  ) -> v == uncamelize (show v')
                               | Just v' <- (fromValue x :: Maybe String   ) -> v  == v'
                               | Just v' <- (fromValue x :: Maybe [String] ) -> v `elem` v'
-                              | Just v' <- (fromValue x :: Maybe [Agent]  ) -> v == [] && v' == [] || v == show v'
-                              | Just v' <- (fromValue x :: Maybe [RefDate]) -> v == [] && v' == [] || v == show v'
+                              | Just v' <- (fromValue x :: Maybe [Agent]  ) -> null v && null v' || v == show v'
+                              | Just v' <- (fromValue x :: Maybe [RefDate]) -> null v && null v' || v == show v'
                        _                                                    -> False
 
 -- | Given the CSL 'Style' and the list of 'Cite's coupled with their
 -- 'Reference's, generate a 'CitationGroup'. The citations are sorted
 -- according to the 'Style'.
-procGroup :: Style -> [(Cite, Reference)] -> CitationGroup
+procGroup :: Style -> [(Cite, Maybe Reference)] -> CitationGroup
 procGroup (Style {citation = ct, csMacros = ms , styleLocale = l,
                   styleAbbrevs = as, csOptions = opts}) cr
     = CG authIn (layFormat $ citLayout ct) (layDelim $ citLayout ct) (authIn ++ co)
     where
       (co, authIn) = case cr of
                        (c:_) -> if authorInText (fst c)
-                                then (filter (eqCites (/=) c) $ result,
+                                then (filter (eqCites (/=) c) result,
                                       take 1 .  filter (eqCites (==) c) $ result)
                                 else (result, [])
                        _     -> (result, [])
@@ -278,7 +308,7 @@ formatCitLayout :: Style -> CitationGroup -> Formatted
 formatCitLayout s (CG co f d cs)
     | [a] <- co = combine (formatAuth a)
                   (formatCits $
-                   (fst >>> citeId &&& citeHash >>> setAsSupAu $ a) $ cs)
+                   (fst >>> citeId &&& citeHash >>> setAsSupAu $ a) cs)
     | otherwise = formatCits cs
     where
       isNote    = styleClass s == "note"
@@ -295,12 +325,17 @@ formatCitLayout s (CG co f d cs)
                      formatOutputList . appendOutput formatting . addAffixes f .
                      addDelim d .
                      map (fst &&& localMod >>> uncurry addCiteAffixes)
-      formatting   = unsetAffixes f
+      formatting   = f{ prefix = [], suffix = [],
+                        verticalAlign = if isAuthorInText cs
+                                           then ""
+                                           else verticalAlign f }
+      isAuthorInText [] = False
+      isAuthorInText ((c,_):_) = authorInText c
       localMod     = uncurry $ localModifiers s (not $ null co)
       setAsSupAu h = map $ \(c,o) -> if (citeId c, citeHash c) == h
-                                     then flip (,) o c { authorInText   = False
-                                                       , suppressAuthor = True }
-                                     else flip (,) o c
+                                     then (c { authorInText   = False
+                                             , suppressAuthor = True }, o)
+                                     else (c, o)
 
 addAffixes :: Formatting -> [Output] -> [Output]
 addAffixes f os
@@ -310,7 +345,7 @@ addAffixes f os
     | otherwise                = pref ++ suff
     where
       pref = if not (null (prefix f))
-             then [OStr (prefix f) emptyFormatting] ++ os
+             then OStr (prefix f) emptyFormatting : os
              else os
       suff = case suffix f of
                   []     -> []
@@ -323,7 +358,7 @@ addAffixes f os
 -- pandoc terminology).
 localModifiers :: Style -> Bool -> Cite -> Output -> Output
 localModifiers s b c
-    | authorInText   c = check . return . proc rmFormatting . contribOnly s
+    | authorInText   c = check . return . contribOnly s
     | suppressAuthor c = check . rmContrib . return
     | otherwise        = id
     where
@@ -352,10 +387,6 @@ localModifiers s b c
           | OStr    x f <- o  = OStr x (if isPunct' (prefix f)
                                         then f { prefix = []} else f) : os
           | otherwise         = o:os
-      rmFormatting f
-          | Formatting {} <- f = emptyFormatting { prefix = prefix f
-                                                 , suffix = suffix f}
-          | otherwise          = f
       rmCitNum o
           | OCitNum {} <- o = ONull
           | otherwise       = o
@@ -388,19 +419,19 @@ localModifiers s b c
 contribOnly :: Style -> Output -> Output
 contribOnly s o
     | isNumStyle [o]
-    , OCitNum  {} <- o = Output [ OStr (query getRefTerm s) emptyFormatting
-                                , OSpace, o] emptyFormatting
+    , OCitNum n f <- o = Output [ OCitNum n f{
+                                       verticalAlign = "",
+                                       prefix = "",
+                                       suffix = "" } ] emptyFormatting
     | OContrib _ "author"
             _ _ _ <- o = o
     | OContrib _ "authorsub"
             _ _ _ <- o = o
-    | Output ot f <- o = Output (cleanOutput $ map (contribOnly s) ot) f
+    | Output ot f <- o = Output (cleanOutput $ map (contribOnly s) ot)
+                         f{ verticalAlign = "",
+                            prefix = "",
+                            suffix = "" }
     | OStr    x _ <- o
     , "ibid" <- filter (/= '.')
        (map toLower x) = o
     | otherwise        = ONull
-    where
-      getRefTerm :: CslTerm -> String
-      getRefTerm t
-          | CT "reference" Long _ _ x _ _ <- t = capitalize x
-          | otherwise                          = []
