@@ -60,7 +60,7 @@ disambCitations s bibs cs groups
                    then if isByCite then ByCite else AllNames
                    else NoGiven
 
-      clean     = if hasGNameOpt then id else proc rmNameHash . proc rmGivenNames
+      clean     = if hasGNameOpt then id else proc rmHashAndGivenNames
       withNames = flip map duplics $ same . clean .
                   map (if hasNamesOpt then disambData else return . disambYS)
 
@@ -79,7 +79,7 @@ disambCitations s bibs cs groups
       reEval      = let chk = if hasYSuffOpt then filter ((==) [] . citYear) else id
                     in  chk needYSuff
       reEvaluated = if or (query hasIfDis s) && not (null reEval)
-                    then map (uncurry $ reEvaluate s reEval) $ zip refs groups
+                    then zipWith (reEvaluate s reEval) refs groups
                     else groups
 
       withYearS = addNames $
@@ -111,7 +111,7 @@ data GiveNameDisambiguation
 disambAddNames :: GiveNameDisambiguation -> [CiteData] -> [CiteData]
 disambAddNames b needName = addLNames
     where
-      clean     = if b == NoGiven then proc rmNameHash . proc rmGivenNames else id
+      clean     = if b == NoGiven then proc rmHashAndGivenNames else id
       disSolved = zip needName' . disambiguate . map disambData $ needName'
       needName' = nub' needName []
       addLNames = map (\(c,n) -> c { disambed = if null n then collision c else head n }) disSolved
@@ -136,7 +136,7 @@ updateContrib g c n o
                                     | otherwise             -> o
     | otherwise = o
     where
-      clean         = if g == NoGiven then proc rmNameHash . proc rmGivenNames else id
+      clean         = if g == NoGiven then proc rmHashAndGivenNames else id
       processGNames = if g /= NoGiven then updateOName n else id
 
 updateOName :: [NameData] -> Output -> Output
@@ -152,14 +152,17 @@ updateOName n o
 -- | Evaluate again a citation group with the 'EvalState' 'disamb'
 -- field set to 'True' (for matching the @\"disambiguate\"@
 -- condition).
-reEvaluate :: Style -> [CiteData] -> [(Cite, Reference)] -> CitationGroup -> CitationGroup
+reEvaluate :: Style -> [CiteData] -> [(Cite, Maybe Reference)] -> CitationGroup -> CitationGroup
 reEvaluate (Style {citation = ct, csMacros = ms , styleLocale = lo,
                    styleAbbrevs = as}) l cr (CG a f d os)
     = CG a f d . flip concatMap (zip cr os) $
-      \((c,r),out) -> if unLiteral (refId r) `elem` map key l
-                      then return . second (flip Output emptyFormatting) $
-                           (,) c $ evalLayout (citLayout ct) (EvalCite c) True lo ms (citOptions ct) as r
-                      else [out]
+      \((c,mbr),out) ->
+         case mbr of
+              Just r | unLiteral (refId r) `elem` lkeys ->
+                return . second (flip Output emptyFormatting) $
+                      (,) c $ evalLayout (citLayout ct) (EvalCite c) True lo ms (citOptions ct) as mbr
+              _ -> [out]
+        where lkeys = map key l
 
 -- | Check if the 'Style' has any conditional for disambiguation. In
 -- this case the conditional will be try after all other
@@ -189,7 +192,7 @@ getDuplCiteData b1 b2 g
       $ duplicates
     where
       whatToGet  = if b1 then collision else disambYS
-      collide    = proc rmExtras . proc rmNameHash . proc rmGivenNames . whatToGet
+      collide    = proc rmExtras . proc rmHashAndGivenNames . whatToGet
       citeData   = nubBy (\a b -> collide a == collide b && key a == key b) $
                    concatMap (mapGroupOutput $ getCiteData) g
       duplicates = [c | c <- citeData , d <- citeData , collides c d]
@@ -220,8 +223,9 @@ getCiteData out
                         [] -> [CD [] [out] [] [] [] [] []]
                               -- allow title to disambiguate
                         xs -> xs
-      yearsQ  = query getYears
-      years o = if yearsQ o /= [] then yearsQ o else [([],[])]
+      years o = case query getYears o of
+                     []    -> [([],[])]
+                     r     -> r
       zipData = uncurry . zipWith $ \c y -> if key c /= []
                                             then c {citYear = snd y}
                                             else c {key     = fst y
@@ -387,16 +391,14 @@ hasYearSuf = not . null . query getYearSuf
               | OYearSuf _ _ _ _ <- o = ["a"]
               | otherwise             = []
 
--- | Removes all given names form a 'OName' element with 'proc'.
-rmGivenNames :: Output -> Output
-rmGivenNames o
-    | OName i s _ f <- o = OName i s [] f
-    | otherwise          = o
+-- | Removes all given names and name hashes from OName elements.
+rmHashAndGivenNames :: Output -> Output
+rmHashAndGivenNames (OName _ s _ f) = OName emptyAgent s [] f
+rmHashAndGivenNames o = o
 
-rmNameHash :: Output -> Output
-rmNameHash o
-    | OName _ s ss f <- o = OName emptyAgent s ss f
-    | otherwise           = o
+rmGivenNames :: Output -> Output
+rmGivenNames (OName a s _ f) = OName a s [] f
+rmGivenNames o = o
 
 -- | Add, with 'proc', a give name to the family name. Needed for
 -- disambiguation.
