@@ -20,7 +20,6 @@ module Text.CSL.Reference ( Literal(..)
                           , Value(..)
                           , ReferenceMap
                           , mkRefMap
-                          , formatField
                           , fromValue
                           , isValueSet
                           , Empty(..)
@@ -46,18 +45,25 @@ import Data.List.Split ( splitWhen )
 import Data.Maybe ( fromMaybe             )
 import Data.Generics hiding (Generic)
 import GHC.Generics (Generic)
-import Data.Monoid
 import Data.Aeson hiding (Value)
+import qualified Data.Aeson as Aeson
 import Data.Aeson.Types (Parser)
-import Control.Applicative ((<$>), (<*>), (<|>), pure)
+import qualified Data.Yaml.Builder as Y
+import Data.Yaml.Builder (ToYaml(..))
+import Control.Applicative ((<|>))
 import qualified Data.Text as T
+import Data.Text (Text)
 import qualified Data.Vector as V
-import Data.Char (toLower, isUpper, isLower, isDigit)
+import Data.Char (toLower, isDigit)
 import Text.CSL.Style hiding (Number)
 import Text.CSL.Util (parseString, parseInt, parseBool, safeRead, readNum,
-                      inlinesToString, capitalize, camelize)
+                      inlinesToString, capitalize, camelize, uncamelize,
+                      (&=), mapping')
 import Text.Pandoc (Inline(Str))
 import Data.String
+import qualified Text.Parsec as P
+import qualified Text.Parsec.String as P
+import qualified Data.HashMap.Strict as H
 
 newtype Literal = Literal { unLiteral :: String }
   deriving ( Show, Read, Eq, Data, Typeable, Monoid, Generic )
@@ -68,6 +74,9 @@ instance FromJSON Literal where
 instance ToJSON Literal where
   toJSON = toJSON . unLiteral
 
+instance ToYaml Literal where
+  toYaml = Y.string . T.pack . unLiteral
+
 instance IsString Literal where
   fromString = Literal
 
@@ -81,15 +90,10 @@ instance Show Value where
 
 type ReferenceMap = [(String, Value)]
 
-mkRefMap :: Data a => a -> ReferenceMap
-mkRefMap a = zip fields (gmapQ Value a)
-    where fields = map formatField . constrFields . toConstr $ a
-
-formatField :: String -> String
-formatField = foldr f [] . g
-    where f  x xs  = if isUpper x then '-' : toLower x : xs else x : xs
-          g (x:xs) = toLower x : xs
-          g     [] = []
+mkRefMap :: Maybe Reference -> ReferenceMap
+mkRefMap Nothing  = []
+mkRefMap (Just r) = zip fields (gmapQ Value r)
+      where fields = map uncamelize . constrFields . toConstr $ r
 
 fromValue :: Data a => Value -> Maybe a
 fromValue (Value a) = cast a
@@ -144,10 +148,19 @@ instance ToJSON RefDate where
     , "month" .= month refdate
     , "season" .= season refdate
     , "day" .= day refdate
-    , "other" .= other refdate ] ++
+    , "literal" .= other refdate ] ++
     [ "circa" .= circa refdate | circa refdate ]
 -}
 
+instance ToYaml RefDate where
+  toYaml r = mapping' [ "year" &= year r
+                      , "month" &= month r
+                      , "season" &= season r
+                      , "day" &= day r
+                      , "literal" &= other r
+                      , "circa" &= T.pack (if circa r then "1" else "")
+                      ]
+
 instance FromJSON [RefDate] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
   parseJSON (Object v) = do
@@ -160,13 +173,13 @@ instance FromJSON [RefDate] where
   parseJSON x          = parseJSON x >>= mkRefDate
 
 -- Zotero doesn't properly support date ranges, so a common
--- workaround is 2005_2007; support this as date range:
+-- workaround is 2005_2007 or 2005_; support this as date range:
 handleLiteral :: RefDate -> [RefDate]
 handleLiteral d@(RefDate (Literal "") (Literal "") (Literal "")
                          (Literal "") (Literal xs) b)
   = case splitWhen (=='_') xs of
          [x,y] | all isDigit x && all isDigit y &&
-                 not (null x) && not (null y) ->
+                 not (null x) ->
                  [RefDate (Literal x) mempty mempty mempty mempty b,
                   RefDate (Literal y) mempty mempty mempty mempty b]
          _ -> [d]
@@ -243,26 +256,29 @@ data RefType
       deriving ( Read, Eq, Typeable, Data, Generic )
 
 instance Show RefType where
-    show MotionPicture = "motion_picture"
-    show MusicalScore = "musical_score"
-    show PersonalCommunication = "personal_communication"
-    show LegalCase = "legal_case"
-    show x = map toLower . formatField . showConstr . toConstr $ x
+    -- TODO: clean up this mess. For now, commenting these out makes
+    -- motion pictures work properly again.
+    -- show MotionPicture = "motion_picture"
+    -- show MusicalScore = "musical_score"
+    -- show PersonalCommunication = "personal_communication"
+    -- show LegalCase = "legal_case"
+    show x = map toLower . uncamelize . showConstr . toConstr $ x
 
 instance FromJSON RefType where
-  parseJSON (String t) = safeRead (capitalize . camelize . T.unpack $ t)
-  parseJSON v@(Array _) = fmap (capitalize . camelize . inlinesToString)
-    (parseJSON v) >>= safeRead
+  parseJSON (String t) =
+    (safeRead (capitalize . camelize . T.unpack $ t)) <|>
+    fail ("'" ++ T.unpack t ++ "' is not a valid reference type")
+  parseJSON v@(Array _) =
+    fmap (capitalize . camelize . inlinesToString) (parseJSON v) >>= \t ->
+      (safeRead t <|>
+       fail ("'" ++ t ++ "' is not a valid reference type"))
   parseJSON _ = fail "Could not parse RefType"
 
 instance ToJSON RefType where
-  toJSON reftype = toJSON (uncamelize $ uncapitalize $ show reftype)
-   where uncamelize [] = []
-         uncamelize (x:y:zs)
-          | isLower x && isUpper y = x:'-':toLower y:uncamelize zs
-         uncamelize (x:xs) = x : uncamelize xs
-         uncapitalize (x:xs) = toLower x : xs
-         uncapitalize []     = []
+  toJSON reftype = toJSON (uncamelize $ show reftype)
+
+instance ToYaml RefType where
+  toYaml r = Y.string (T.pack $ uncamelize $ show r)
 
 newtype CNum = CNum { unCNum :: Int } deriving ( Show, Read, Eq, Num, Typeable, Data, Generic )
 
@@ -272,6 +288,9 @@ instance FromJSON CNum where
 instance ToJSON CNum where
   toJSON (CNum n) = toJSON n
 
+instance ToYaml CNum where
+  toYaml r = Y.string (T.pack $ show $ unCNum r)
+
 -- | The 'Reference' record.
 data Reference =
     Reference
@@ -357,7 +376,9 @@ data Reference =
     } deriving ( Eq, Show, Read, Typeable, Data, Generic )
 
 instance FromJSON Reference where
-  parseJSON (Object v) = addPageFirst <$> (Reference <$>
+  parseJSON (Object v') = do
+     v <- parseSuppFields v'
+     addPageFirst <$> (Reference <$>
        v .:? "id" .!= "" <*>
        v .:? "type" .!= NoType <*>
        v .:? "author" .!= [] <*>
@@ -431,7 +452,7 @@ instance FromJSON Reference where
        v .:? "categories" .!= [] <*>
        v .:? "language" .!= "" <*>
        v .:? "citation-number" .!= CNum 0 <*>
-       ((v .: "first-reference-note-number" >>= parseInt) <|> return 1) <*>
+       ((v .: "first-reference-note-number" >>= parseInt) <|> return 0) <*>
        v .:? "citation-label" .!= "")
     where takeFirstNum (Formatted (Str xs : _)) =
             case takeWhile isDigit xs of
@@ -444,6 +465,32 @@ instance FromJSON Reference where
                                 else ref
   parseJSON _ = fail "Could not parse Reference"
 
+-- Syntax for adding supplementary fields in note variable
+-- {:authority:Superior Court of California}{:section:A}{:original-date:1777}
+-- see http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#supplementary-fields
+parseSuppFields :: Aeson.Object -> Parser Aeson.Object
+parseSuppFields o = do
+  nt <- o .: "note" <|> return ("" :: String)
+  case P.parse noteFields "note" nt of
+       Left err -> fail (show err)
+       Right fs -> return $ foldr (\(k,v) x -> H.insert k v x) o fs
+
+noteFields :: P.Parser [(Text, Aeson.Value)]
+noteFields = do
+  fs <- P.many noteField
+  P.spaces
+  rest <- P.getInput
+  return (("note", Aeson.String (T.pack rest)) : fs)
+
+noteField :: P.Parser (Text, Aeson.Value)
+noteField = P.try $ do
+  P.spaces
+  P.char '{'
+  P.char ':'
+  k <- P.manyTill (P.letter <|> P.char '-') (P.char ':')
+  v <- P.manyTill P.anyChar (P.char '}')
+  return (T.pack k, Aeson.String (T.pack v))
+
 instance ToJSON Reference where
   toJSON ref = object' [
       "id" .= refId ref
@@ -523,6 +570,90 @@ instance ToJSON Reference where
     , "citation-label" .= citationLabel ref
     ]
 
+instance ToYaml Reference where
+  toYaml ref = mapping' [
+      "id" &= refId ref
+    , (("type" Y..= refType ref) :)
+    , "author" &= author ref
+    , "editor" &= editor ref
+    , "translator" &= translator ref
+    , "recipient" &= recipient ref
+    , "interviewer" &= interviewer ref
+    , "composer" &= composer ref
+    , "director" &= director ref
+    , "illustrator" &= illustrator ref
+    , "original-author" &= originalAuthor ref
+    , "container-author" &= containerAuthor ref
+    , "collection-editor" &= collectionEditor ref
+    , "editorial-director" &= editorialDirector ref
+    , "reviewed-author" &= reviewedAuthor ref
+    , "issued" &= issued ref
+    , "event-date" &= eventDate ref
+    , "accessed" &= accessed ref
+    , "container" &= container ref
+    , "original-date" &= originalDate ref
+    , "submitted" &= submitted ref
+    , "title" &= title ref
+    , "title-short" &= titleShort ref
+    , "reviewed-title" &= reviewedTitle ref
+    , "container-title" &= containerTitle ref
+    , "volume-title" &= volumeTitle ref
+    , "collection-title" &= collectionTitle ref
+    , "container-title-short" &= containerTitleShort ref
+    , "collection-number" &= collectionNumber ref
+    , "original-title" &= originalTitle ref
+    , "publisher" &= publisher ref
+    , "original-publisher" &= originalPublisher ref
+    , "publisher-place" &= publisherPlace ref
+    , "original-publisher-place" &= originalPublisherPlace ref
+    , "authority" &= authority ref
+    , "jurisdiction" &= jurisdiction ref
+    , "archive" &= archive ref
+    , "archive-place" &= archivePlace ref
+    , "archive_location" &= archiveLocation ref
+    , "event" &= event ref
+    , "event-place" &= eventPlace ref
+    , "page" &= page ref
+    , "page-first" &= (if page ref == mempty then pageFirst ref else mempty)
+    , "number-of-pages" &= numberOfPages ref
+    , "version" &= version ref
+    , "volume" &= volume ref
+    , "number-of-volumes" &= numberOfVolumes ref
+    , "issue" &= issue ref
+    , "chapter-number" &= chapterNumber ref
+    , "medium" &= medium ref
+    , "status" &= status ref
+    , "edition" &= edition ref
+    , "section" &= section ref
+    , "source" &= source ref
+    , "genre" &= genre ref
+    , "note" &= note ref
+    , "annote" &= annote ref
+    , "abstract" &= abstract ref
+    , "keyword" &= keyword ref
+    , "number" &= number ref
+    , "references" &= references ref
+    , "URL" &= url ref
+    , "DOI" &= doi ref
+    , "ISBN" &= isbn ref
+    , "ISSN" &= issn ref
+    , "PMCID" &= pmcid ref
+    , "PMID" &= pmid ref
+    , "call-number" &= callNumber ref
+    , "dimensions" &= dimensions ref
+    , "scale" &= scale ref
+    , "categories" &= categories ref
+    , "language" &= language ref
+    , if citationNumber ref == CNum 0
+         then id
+         else (("citation-number" Y..= citationNumber ref) :)
+    , if firstReferenceNoteNumber ref == 0
+         then id
+         else (("first-reference-note-number" Y..=
+                firstReferenceNoteNumber ref) :)
+    , "citation-label" &= citationLabel ref
+    ]
+
 emptyReference :: Reference
 emptyReference =
     Reference
@@ -617,17 +748,13 @@ getReference  r c
         Just i  -> Just $ setPageFirst $ r !! i
         Nothing -> Nothing
 
-processCites :: [Reference] -> [[Cite]] -> [[(Cite, Reference)]]
+processCites :: [Reference] -> [[Cite]] -> [[(Cite, Maybe Reference)]]
 processCites rs cs
     = procGr [[]] cs
     where
       procRef r = case filter ((==) (unLiteral $ refId r) . citeId) $ concat cs of
                     x:_ -> r { firstReferenceNoteNumber = readNum $ citeNoteNumber x}
                     []  -> r
-      getRef  c = case filter ((==) (citeId c) . unLiteral . refId) rs of
-                    x:_ -> procRef $ setPageFirst x
-                    []  -> emptyReference { title =
-                            fromString $ citeId c ++ " not found!" }
 
       procGr _ [] = []
       procGr a (x:xs) = let (a',res) = procCs a x
@@ -642,7 +769,8 @@ processCites rs cs
           where
             go s = let addCite    = init a ++ [last a ++ [c]]
                        (a', rest) = procCs addCite xs
-                   in  (a', (c { citePosition = s}, getRef c) : rest)
+                   in  (a', (c { citePosition = s},
+                             procRef <$> getReference rs c) : rest)
             isElem   = citeId c `elem` map citeId (concat a)
             isIbid   = case reverse (last a) of
                             []    -> case reverse (init a) of
