@@ -1,6 +1,6 @@
 {-# LANGUAGE OverloadedStrings, PatternGuards, DeriveDataTypeable,
     ScopedTypeVariables, FlexibleInstances, DeriveGeneric,
-    GeneralizedNewtypeDeriving #-}
+    GeneralizedNewtypeDeriving, CPP, MultiParamTypeClasses #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Style
@@ -59,7 +59,6 @@ module Text.CSL.Style ( readCSLString
                       , emptyFormatting
                       , rmTitleCase
                       , Quote(..)
-                      , unsetAffixes
                       , mergeFM
                       , CSInfo(..)
                       , CSAuthor(..)
@@ -83,9 +82,9 @@ where
 import Data.Aeson hiding (Number)
 import GHC.Generics (Generic)
 import Data.String
-import Data.Monoid (mempty, Monoid, mappend, mconcat, (<>))
 import Control.Arrow hiding (left, right)
-import Control.Applicative hiding (Const)
+import Control.Monad (liftM, mplus)
+import Control.Applicative ((<|>))
 import qualified Data.Aeson as Aeson
 import Data.Aeson.Types (Pair)
 import Data.List ( nubBy, isPrefixOf, isInfixOf, intersperse, intercalate )
@@ -96,10 +95,14 @@ import qualified Data.Map as M
 import Data.Char (isPunctuation, isUpper, isLetter)
 import Text.CSL.Util (mb, parseBool, parseString, (.#?), (.#:), query,
                       betterThan, trimr, tailInline, headInline,
-                      initInline, lastInline, splitStrWhen)
+                      initInline, lastInline, splitStrWhen, mapping',
+                      (&=))
+import Data.Yaml.Builder(ToYaml(..))
+import qualified Data.Yaml.Builder as Y
 import Text.Pandoc.Definition hiding (Citation, Cite)
-import Text.Pandoc (readHtml, writeMarkdown, WriterOptions(..),
-                    ReaderOptions(..), bottomUp, def)
+import Text.Pandoc (readHtml, writeMarkdown,
+                    WriterOptions(..), ReaderOptions(..), WrapOption(..),
+                    bottomUp, def)
 import qualified Text.Pandoc.Walk as Walk
 import qualified Text.Pandoc.Builder as B
 import qualified Data.Text as T
@@ -124,9 +127,16 @@ readCSLString s = Walk.walk handleSmallCapsSpans
                 $ case readHtml def{ readerSmart = True
                                    , readerParseRaw = True }
                                 (adjustScTags s) of
+#if MIN_VERSION_pandoc(1,14,0)
+                        Right (Pandoc _ [Plain ils])   -> ils
+                        Right (Pandoc _ [Para  ils])   -> ils
+                        Right (Pandoc _ x)             -> Walk.query (:[]) x
+                        Left  _                        -> []
+#else
                         Pandoc _ [Plain ils]   -> ils
                         Pandoc _ [Para  ils]   -> ils
                         Pandoc _ x             -> Walk.query (:[]) x
+#endif
   -- this is needed for versions of pandoc that don't turn
   -- a span with font-variant:small-caps into a SmallCaps element:
   where handleSmallCapsSpans (Span ("",[],[("style",sty)]) ils)
@@ -146,25 +156,41 @@ adjustScTags zs =
        []                       -> []
 
 
+writeYAMLString :: [Inline] -> String
+writeYAMLString ils =
+  trimr $ writeMarkdown def{writerWrapText = WrapNone}
+        $ Pandoc nullMeta
+          [Plain $ bottomUp (concatMap (adjustCSL False)) ils]
+
 writeCSLString :: [Inline] -> String
 writeCSLString ils =
-  trimr $ writeMarkdown def{writerWrapText = False}
-        $ Pandoc nullMeta [Plain $ bottomUp (concatMap adjustCSL) ils]
+  trimr $ writeMarkdown def{writerWrapText = WrapNone}
+        $ Pandoc nullMeta
+          [Plain $ bottomUp (concatMap (adjustCSL True)) ils]
 
-adjustCSL :: Inline -> [Inline]
-adjustCSL (Span ("",[],[]) xs) = xs
-adjustCSL (Span ("",["citeproc-no-output"],[]) _) =
+-- If the first param is True, we use special rich text conventions
+-- for CSL JSON, described here:
+-- http://docs.citationstyles.org/en/1.0/release-notes.html#rich-text-markup-within-fields
+adjustCSL :: Bool -> Inline -> [Inline]
+adjustCSL _ (Span ("",[],[]) xs) = xs
+adjustCSL _ (Span ("",["citeproc-no-output"],[]) _) =
   [Str "[CSL STYLE ERROR: reference with no printed form.]"]
-adjustCSL (SmallCaps xs) =
-  RawInline (Format "html") "<span style=\"font-variant:small-caps;\">" : xs
-    ++ [RawInline (Format "html") "</span>"]
-adjustCSL (Subscript xs) =
-  RawInline (Format "html") "<span style=\"vertical-align:sub;\">" : xs
-    ++ [RawInline (Format "html") "</span>"]
-adjustCSL (Superscript xs) =
-  RawInline (Format "html") "<span style=\"vertical-align:sup;\">" : xs
-    ++ [RawInline (Format "html") "</span>"]
-adjustCSL x = [x]
+adjustCSL True (SmallCaps xs) =
+  RawInline (Format "html") "<sc>" : xs
+    ++ [RawInline (Format "html") "</sc>"]
+adjustCSL True (Subscript xs) =
+  RawInline (Format "html") "<sub>" : xs
+    ++ [RawInline (Format "html") "</sub>"]
+adjustCSL True (Superscript xs) =
+  RawInline (Format "html") "<sup>" : xs
+    ++ [RawInline (Format "html") "</sup>"]
+adjustCSL True (Emph xs) =
+  RawInline (Format "html") "<i>" : xs
+    ++ [RawInline (Format "html") "</i>"]
+adjustCSL True (Strong xs) =
+  RawInline (Format "html") "<b>" : xs
+    ++ [RawInline (Format "html") "</b>"]
+adjustCSL _ x = [x]
 
 -- We use a newtype wrapper so we can have custom ToJSON, FromJSON
 -- instances.
@@ -180,6 +206,9 @@ instance FromJSON Formatted where
 instance ToJSON Formatted where
   toJSON = toJSON . writeCSLString . unFormatted
 
+instance ToYaml Formatted where
+  toYaml = Y.string . T.pack . writeYAMLString . unFormatted
+
 instance IsString Formatted where
   fromString = Formatted . toStr
 
@@ -188,6 +217,16 @@ instance Monoid Formatted where
   mappend = appendWithPunct
   mconcat = foldr mappend mempty
 
+instance Walk.Walkable Inline Formatted where
+  walk f  = Formatted . Walk.walk f . unFormatted
+  walkM f = liftM Formatted . Walk.walkM f . unFormatted
+  query f = Walk.query f . unFormatted
+
+instance Walk.Walkable Formatted Formatted where
+  walk f  = f
+  walkM f = f
+  query f = f
+
 toStr :: String -> [Inline]
 toStr = intercalate [Str "\n"] .
         map (B.toList . B.text . tweak . fromEntities) .
@@ -280,12 +319,23 @@ newTerm :: CslTerm
 newTerm = CT [] Long Neuter Neuter [] [] []
 
 findTerm :: String -> Form -> [CslTerm] -> Maybe CslTerm
-findTerm s f
-    = listToMaybe . filter (cslTerm &&& termForm >>> (==) (s, f))
+findTerm s f = findTerm'' s f Nothing
 
 findTerm' :: String -> Form -> Gender -> [CslTerm] -> Maybe CslTerm
-findTerm' s f g
-    = listToMaybe . filter (cslTerm &&& termForm &&& termGenderForm >>> (==) (s,(f,g)))
+findTerm' s f g = findTerm'' s f (Just g)
+
+findTerm'' :: String -> Form -> Maybe Gender -> [CslTerm] -> Maybe CslTerm
+findTerm'' s f mbg ts
+  = listToMaybe [ t | t <- ts, cslTerm t == s, termForm t == f,
+                         mbg == Nothing || mbg == Just (termGenderForm t) ]
+  `mplus`
+  -- fallback: http://citationstyles.org/downloads/specification.html#terms
+  case f of
+       VerbShort -> findTerm'' s Verb Nothing ts
+       Symbol    -> findTerm'' s Short Nothing ts
+       Verb      -> findTerm'' s Long Nothing ts
+       Short     -> findTerm'' s Long Nothing ts
+       _         -> Nothing
 
 hasOrdinals :: [Locale] -> Bool
 hasOrdinals = any (any hasOrd . localeTerms)
@@ -513,6 +563,7 @@ data Formatting
       , stripPeriods   :: Bool
       , noCase         :: Bool
       , noDecor        :: Bool
+      , hyperlink      :: String  -- null for no link
       } deriving ( Read, Eq, Ord, Typeable, Data, Generic )
 
 -- custom instance to make debugging output less busy
@@ -535,7 +586,8 @@ instance Show Formatting where
                        ,("quotes", show . quotes)
                        ,("stripPeriods", show . stripPeriods)
                        ,("noCase", show . noCase)
-                       ,("noDecor", show . noDecor)],
+                       ,("noDecor", show . noDecor)
+                       ,("hyperlink", show . hyperlink)],
              f x /= f emptyFormatting ]
         ++ "}"
 
@@ -550,14 +602,11 @@ data Quote
 
 emptyFormatting :: Formatting
 emptyFormatting
-    = Formatting [] [] [] [] [] [] [] [] [] [] NoQuote False False False
-
-unsetAffixes :: Formatting -> Formatting
-unsetAffixes f = f {prefix = [], suffix = []}
+    = Formatting [] [] [] [] [] [] [] [] [] [] NoQuote False False False []
 
 mergeFM :: Formatting -> Formatting -> Formatting
-mergeFM (Formatting aa ab ac ad ae af ag ah ai aj ak al am an)
-        (Formatting ba bb bc bd be bf bg bh bi bj bk bl bm bn) =
+mergeFM (Formatting aa ab ac ad ae af ag ah ai aj ak al am an ahl)
+        (Formatting ba bb bc bd be bf bg bh bi bj bk bl bm bn bhl) =
                    Formatting (ba `betterThan` aa)
                               (bb `betterThan` ab)
                               (bc `betterThan` ac)
@@ -572,6 +621,7 @@ mergeFM (Formatting aa ab ac ad ae af ag ah ai aj ak al am an)
                               (bl || al)
                               (bm || am)
                               (bn || an)
+                              (bhl `mplus` ahl)
 
 data CSInfo
     = CSInfo
@@ -663,6 +713,7 @@ data BiblioData
     = BD
       { citations    :: [Formatted]
       , bibliography :: [Formatted]
+      , citationIds  :: [String]
       } deriving ( Show, Typeable, Data, Generic )
 
 -- | A record with all the data to produce the 'Formatted' of a
@@ -730,37 +781,64 @@ emptyAgent = Agent [] mempty mempty mempty mempty mempty False False
 
 instance FromJSON Agent where
   parseJSON (Object v) = nameTransform <$> (Agent <$>
-              (v .: "given" <|> ((map Formatted . wordsBy (== Space) . unFormatted) <$> v .: "given") <|> pure []) <*>
+              (v .: "given" <|> ((map Formatted . wordsBy isSpace . unFormatted) <$> v .: "given") <|> pure []) <*>
               v .:?  "dropping-particle" .!= mempty <*>
               v .:? "non-dropping-particle" .!= mempty <*>
               v .:? "family" .!= mempty <*>
               v .:? "suffix" .!= mempty <*>
               v .:? "literal" .!= mempty <*>
               v .:? "comma-suffix" .!= False <*>
-              v .:? "parse-names" .!= True)
+              v .:? "parse-names" .!= False)
   parseJSON _ = fail "Could not parse Agent"
 
+instance ToYaml Agent where
+  toYaml ag = mapping' [ "family" &= familyName ag
+                       , case givenName ag of
+                              []    -> id
+                              xs    -> "given" &= Formatted
+                                                  (intercalate [Space]
+                                                    (map unFormatted xs))
+                       , "non-dropping-particle" &= nonDroppingPart ag
+                       , "dropping-particle" &= droppingPart ag
+                       , "suffix" &= nameSuffix ag
+                       , "literal" &= literal ag
+                       , "comma-suffix" &= T.pack (if commaSuffix ag
+                                                      then "true"
+                                                      else "")
+                       , "parse-names" &= T.pack (if parseNames ag
+                                                     then "true"
+                                                     else "")
+                       ]
+
 -- See http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#id28
 nameTransform :: Agent -> Agent
 nameTransform ag
   | parseNames ag = nonDroppingPartTransform .
                     droppingPartTransform .
-                    suffixTransform $ ag
+                    suffixTransform $ ag{ parseNames = False }
   | otherwise = ag
 
 nonDroppingPartTransform :: Agent -> Agent
 nonDroppingPartTransform ag
   | nonDroppingPart ag == mempty =
-    case break startWithCapital' (unFormatted $ familyName ag) of
+    case break startWithCapital'
+         (splitStrWhen (\c -> isPunctuation c || isUpper c) $
+          unFormatted $ familyName ag) of
          ([], _)  -> ag
-         (xs, ys) -> ag { nonDroppingPart = Formatted $ trimSpace xs,
+         (xs, ys)
+           | lastInline xs `elem` [" ", "-", "'", "’"] -> ag {
+                          nonDroppingPart = Formatted $ trimSpace xs,
                           familyName = Formatted ys }
+           | otherwise -> ag
   | otherwise = ag
 
 trimSpace :: [Inline] -> [Inline]
 trimSpace = reverse . dropWhile isSpace . reverse . dropWhile isSpace
-  where isSpace Space = True
-        isSpace _     = False
+
+isSpace :: Inline -> Bool
+isSpace Space     = True
+isSpace SoftBreak = True
+isSpace _         = False
 
 droppingPartTransform :: Agent -> Agent
 droppingPartTransform ag
@@ -818,7 +896,7 @@ instance ToJSON Agent where
     , "suffix" .= nameSuffix agent
     , "literal" .= literal agent
     ] ++ ["comma-suffix" .= commaSuffix agent | nameSuffix agent /= mempty]
-      ++ ["parse-names" .= False | not (parseNames agent) ]
+      ++ ["parse-names" .= True | parseNames agent ]
 
 instance FromJSON [Agent] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
