@@ -5,6 +5,7 @@ module Text.CSL.Util
   , (<^>)
   , capitalize
   , camelize
+  , uncamelize
   , isPunct
   , last'
   , init'
@@ -28,8 +29,6 @@ module Text.CSL.Util
   , procM
   , query
   , betterThan
-  , readable
-  , toShow
   , toRead
   , inlinesToString
   , headInline
@@ -41,25 +40,41 @@ module Text.CSL.Util
   , mapHeadInline
   , tr'
   , findFile
+  , (&=)
+  , mapping'
+  , parseRomanNumeral
   ) where
 import Data.Aeson
 import Data.Aeson.Types (Parser)
 import Data.Text (Text)
 import qualified Data.Text as T
-import Control.Applicative ((<$>), (<*>), pure)
-import Data.Char (toLower, toUpper, isLower, isUpper, isPunctuation)
+import Data.Char (toLower, toUpper, isLower, isUpper, isPunctuation,
+                  isLetter, isAscii)
 import qualified Data.Traversable
 import Text.Pandoc.Shared (safeRead, stringify)
 import Text.Pandoc.Walk (walk)
 import Text.Pandoc
 import Data.List.Split (wordsBy)
 import Control.Monad.State
-import Data.Monoid (Monoid, mappend, mempty)
 import Data.Generics ( Typeable, Data, everywhere, everywhereM, mkM,
                        everywhere', everything, mkT, mkQ )
 import System.FilePath
 import System.Directory (doesFileExist)
+import qualified Data.Yaml.Builder as Y
+import Data.Yaml.Builder (ToYaml(..), YamlBuilder)
+import qualified Text.Parsec as P
+
+#ifdef TRACE
 import qualified Debug.Trace
+import Text.Show.Pretty (ppShow)
+#endif
+
+tr' :: Show a => String -> a -> a
+#ifdef TRACE
+tr' note' x = Debug.Trace.trace ("=== " ++ note' ++ "\n" ++ ppShow x ++ "\n") x
+#else
+tr' _ x = x
+#endif
 
 readNum :: String -> Int
 readNum s = case reads s of
@@ -88,6 +103,12 @@ camelize ('_':y:ys) = toUpper y : camelize ys
 camelize     (y:ys) =         y : camelize ys
 camelize      _     = []
 
+uncamelize :: String -> String
+uncamelize = foldr g [] . f
+    where g    x xs  = if isUpper x then '-' : toLower x : xs else x : xs
+          f (  x:xs) = toLower x : xs
+          f       [] = []
+
 last' :: [a] -> [a]
 last' [] = []
 last' xs = [last xs]
@@ -194,9 +215,9 @@ titlecase zs = evalState (caseTransform tc zs) SentenceBoundary
   where tc (Str (x:xs)) = do
           st <- get
           return $ case st of
-                        WordBoundary -> if isShortWord (x:xs)
+                        WordBoundary -> if isShortWord (x:xs) ||
+                                             not (isAscii x && isLetter x)
                                            then Str (x:xs)
-                                                -- or? map toLower (x:xs)
                                            else Str (toUpper x : xs)
                         SentenceBoundary -> Str (toUpper x : xs)
                         _ -> Str (x:xs)
@@ -213,7 +234,7 @@ data CaseTransformState = WordBoundary | SentenceBoundary | NoBoundary
 caseTransform :: (Inline -> State CaseTransformState Inline) -> [Inline]
               -> State CaseTransformState [Inline]
 caseTransform xform = fmap reverse . foldM go [] . splitUpStr
-  where go acc Space        = do
+  where go acc s | s == Space || s == SoftBreak = do
                modify (\st ->
                  case st of
                       SentenceBoundary -> SentenceBoundary
@@ -246,8 +267,8 @@ caseTransform xform = fmap reverse . foldM go [] . splitUpStr
         go acc (Quoted qt xs)    = (:acc) <$> (Quoted qt <$> caseTransform xform xs)
         go acc (Emph xs)         = (:acc) <$> (Emph <$> caseTransform xform xs)
         go acc (Strong xs)       = (:acc) <$> (Strong <$> caseTransform xform xs)
-        go acc (Link xs t)       = (:acc) <$> (Link <$> caseTransform xform xs <*> pure t)
-        go acc (Image xs t)      = (:acc) <$> (Link <$> caseTransform xform xs <*> pure t)
+        go acc (Link at xs t)       = (:acc) <$> (Link at <$> caseTransform xform xs <*> pure t)
+        go acc (Image at xs t)      = (:acc) <$> (Link at <$> caseTransform xform xs <*> pure t)
         go acc (Span attr xs)    = (:acc) <$> (Span attr <$> caseTransform xform xs)
         go acc x                 = return $ x : acc
 
@@ -283,15 +304,6 @@ betterThan :: [a] -> [a] -> [a]
 betterThan [] b = b
 betterThan a  _ = a
 
-readable :: (Read a, Show b) => (String -> a, b -> String)
-readable =  (read . toRead, toShow . show)
-
-toShow :: String -> String
-toShow = foldr g [] . f
-    where g    x xs  = if isUpper x then '-' : toLower x : xs else x : xs
-          f (  x:xs) = toLower x : xs
-          f       [] = []
-
 toRead :: String -> String
 toRead    []  = []
 toRead (s:ss) = toUpper s : camel ss
@@ -324,13 +336,14 @@ initInline (i:[])
     | Quoted q    is <- i = return $ Quoted q    (initInline is)
     | SmallCaps   is <- i = return $ SmallCaps   (initInline is)
     | Strikeout   is <- i = return $ Strikeout   (initInline is)
-    | Link      is t <- i = return $ Link        (initInline is) t
+    | Link   at is t <- i = return $ Link at     (initInline is) t
     | Span at     is <- i = return $ Span at     (initInline is)
     | otherwise           = []
 initInline (i:xs) = i : initInline xs
 
 tailInline :: [Inline] -> [Inline]
 tailInline (Space:xs) = xs
+tailInline (SoftBreak:xs) = xs
 tailInline xs         = removeEmpty $ tailFirstInlineStr xs
   where removeEmpty   = dropWhile (== Str "")
 
@@ -353,13 +366,10 @@ mapHeadInline f (i:xs)
     | Quoted q    is <- i = Quoted q    (mapHeadInline f is)   : xs
     | SmallCaps   is <- i = SmallCaps   (mapHeadInline f is)   : xs
     | Strikeout   is <- i = Strikeout   (mapHeadInline f is)   : xs
-    | Link      is t <- i = Link        (mapHeadInline f is) t : xs
+    | Link   at is t <- i = Link at     (mapHeadInline f is) t : xs
     | Span     at is <- i = Span at     (mapHeadInline f is)   : xs
     | otherwise           = i : xs
 
-tr' :: Show a => String -> a -> a
-tr' note' x = Debug.Trace.trace (note' ++ ": " ++ show x) x
-
 findFile :: [FilePath] -> FilePath -> IO (Maybe FilePath)
 findFile [] _ = return Nothing
 findFile (p:ps) f = do
@@ -368,3 +378,51 @@ findFile (p:ps) f = do
      then return $ Just (p </> f)
      else findFile ps f
 
+(&=) :: (ToYaml a, Monoid a, Eq a)
+     => Text -> a -> [(Text, YamlBuilder)] -> [(Text, YamlBuilder)]
+x &= y = \acc -> if y == mempty
+                    then acc
+                    else (x Y..= y) : acc
+
+mapping' :: [[(Text, YamlBuilder)] -> [(Text, YamlBuilder)]] -> YamlBuilder
+mapping' = Y.mapping . foldr ($) []
+
+-- TODO: romanNumeral is defined in Text.Pandoc.Parsing, but it's
+-- not exported there. Eventually we should remove this code duplication
+-- by exporting something from pandoc.
+
+parseRomanNumeral :: String -> Maybe Int
+parseRomanNumeral s = case P.parse (pRomanNumeral <* P.eof) "" s of
+                           Left _  -> Nothing
+                           Right x -> Just x
+
+-- | Parses a roman numeral (uppercase or lowercase), returns number.
+pRomanNumeral :: P.Stream s m Char => P.ParsecT s st m Int
+pRomanNumeral = do
+    let lowercaseRomanDigits = ['i','v','x','l','c','d','m']
+    let uppercaseRomanDigits = ['I','V','X','L','C','D','M']
+    c <- P.lookAhead $ P.oneOf (lowercaseRomanDigits ++ uppercaseRomanDigits)
+    let romanDigits = if isUpper c
+                         then uppercaseRomanDigits
+                         else lowercaseRomanDigits
+    let [one, five, ten, fifty, hundred, fivehundred, thousand] =
+          map P.char romanDigits
+    thousands <- P.many thousand >>= (return . (1000 *) . length)
+    ninehundreds <- P.option 0 $ P.try $ hundred >> thousand >> return 900
+    fivehundreds <- P.many fivehundred >>= (return . (500 *) . length)
+    fourhundreds <- P.option 0 $ P.try $ hundred >> fivehundred >> return 400
+    hundreds <- P.many hundred >>= (return . (100 *) . length)
+    nineties <- P.option 0 $ P.try $ ten >> hundred >> return 90
+    fifties <- P.many fifty >>= (return . (50 *) . length)
+    forties <- P.option 0 $ P.try $ ten >> fifty >> return 40
+    tens <- P.many ten >>= (return . (10 *) . length)
+    nines <- P.option 0 $ P.try $ one >> ten >> return 9
+    fives <- P.many five >>= (return . (5 *) . length)
+    fours <- P.option 0 $ P.try $ one >> five >> return 4
+    ones <- P.many one >>= (return . length)
+    let total = thousands + ninehundreds + fivehundreds + fourhundreds +
+                hundreds + nineties + fifties + forties + tens + nines +
+                fives + fours + ones
+    if total == 0
+       then fail "not a roman numeral"
+       else return total
