@@ -5,7 +5,6 @@ import System.Exit
 import System.Directory
 import System.FilePath
 import Data.Maybe (fromMaybe)
-import Text.Printf
 import System.IO
 import Data.Monoid (mempty)
 import System.IO.Temp (withSystemTempDirectory)
@@ -18,16 +17,22 @@ import qualified Data.ByteString.Char8 as B
 import qualified Text.Pandoc.UTF8 as UTF8
 import Text.Pandoc.Shared (normalize)
 import Text.Pandoc.Process (pipeProcess)
+import Text.Pandoc.Options (WriterOptions(..))
 import qualified Data.Yaml as Yaml
 import Text.Pandoc (writeNative, writeHtmlString, readNative, def)
 import Text.CSL.Pandoc (processCites')
 import Data.List (isSuffixOf)
+import System.Environment
+import Control.Monad (when)
 
+main :: IO ()
 main = do
+  args <- getArgs
+  let regenerate = args == ["--regenerate"]
   testnames <- fmap (map (dropExtension . takeBaseName) .
                      filter (".in.native" `isSuffixOf`)) $
                getDirectoryContents "tests"
-  citeprocTests <- mapM testCase testnames
+  citeprocTests <- mapM (testCase regenerate) testnames
   fs <- filter (\f -> takeExtension f `elem` [".bibtex",".biblatex"])
            `fmap` getDirectoryContents "tests/biblio2yaml"
   biblio2yamlTests <- mapM biblio2yamlTest fs
@@ -53,17 +58,19 @@ data TestResult =
   | Errored
   deriving (Show, Eq)
 
-testCase :: String -> IO TestResult
-testCase csl = do
+testCase :: Bool -> String -> IO TestResult
+testCase regenerate csl = do
   hPutStr stderr $ "[" ++ csl ++ ".in.native] "
-  indataNative <- readFile $ "tests/" ++ csl ++ ".in.native"
-  expectedNative <- readFile $ "tests/" ++ csl ++ ".expected.native"
+  indataNative <- UTF8.readFile $ "tests/" ++ csl ++ ".in.native"
+  expectedNative <- UTF8.readFile $ "tests/" ++ csl ++ ".expected.native"
   let jsonIn = Aeson.encode $ (read indataNative :: Pandoc)
   let expectedDoc = normalize $ read expectedNative
-
+  testProgPath <- getExecutablePath
+  let pandocCiteprocPath = takeDirectory testProgPath </> ".." </>
+        "pandoc-citeproc" </> "pandoc-citeproc"
   (ec, jsonOut, errout) <- pipeProcess
                      (Just [("LANG","en_US.UTF-8"),("HOME",".")])
-                     "dist/build/pandoc-citeproc/pandoc-citeproc"
+                     pandocCiteprocPath
                      [] jsonIn
   if ec == ExitSuccess
      then do
@@ -72,8 +79,10 @@ testCase csl = do
           then err "PASSED" >> return Passed
           else do
              err $ "FAILED"
-             showDiff (UTF8.fromStringLazy $ writeNative def expectedDoc)
-                      (UTF8.fromStringLazy $ writeNative def outDoc)
+             showDiff (writeNative def expectedDoc) (writeNative def outDoc)
+             when regenerate $
+               UTF8.writeFile ("tests/" ++ csl ++ ".expected.native") $
+                  writeNative def{ writerStandalone = True } outDoc
              return Failed
      else do
        err "ERROR"
@@ -81,13 +90,13 @@ testCase csl = do
        err $ UTF8.toStringLazy errout
        return Errored
 
-showDiff :: BL.ByteString -> BL.ByteString -> IO ()
-showDiff expected' result' =
+showDiff :: String -> String -> IO ()
+showDiff expected result =
   withSystemTempDirectory "test-pandoc-citeproc-XXX" $ \fp -> do
     let expectedf = fp </> "expected"
     let actualf   = fp </> "actual"
-    BL.writeFile expectedf expected'
-    BL.writeFile actualf result'
+    UTF8.writeFile expectedf expected
+    UTF8.writeFile actualf result
     oldDir <- getCurrentDirectory
     setCurrentDirectory fp
     rawSystem "diff" ["-U1","expected","actual"]
@@ -97,28 +106,27 @@ biblio2yamlTest :: String -> IO TestResult
 biblio2yamlTest fp = do
   hPutStr stderr $ "[biblio2yaml/" ++ fp ++ "] "
   let yamlf = "tests/biblio2yaml/" ++ fp
-  raw <- BL.readFile yamlf
-  let yamlStart = BL.pack "---"
-  let (biblines, yamllines) = break (== yamlStart) $ BL.lines raw
-  let bib = BL.unlines biblines
-  let expected = BL.unlines yamllines
-  (ec, result, errout) <- pipeProcess
+  raw <- UTF8.readFile yamlf
+  let yamlStart = "---"
+  let (biblines, yamllines) = break (== yamlStart) $ lines raw
+  let bib = unlines biblines
+  let expected = unlines yamllines
+  testProgPath <- getExecutablePath
+  let pandocCiteprocPath = takeDirectory testProgPath </> ".." </>
+        "pandoc-citeproc" </> "pandoc-citeproc"
+  (ec, result', errout) <- pipeProcess
                      (Just [("LANG","en_US.UTF-8"),("HOME",".")])
-                     "dist/build/pandoc-citeproc/pandoc-citeproc"
-                     ["--bib2yaml", "-f", drop 1 $ takeExtension fp] bib
+                     pandocCiteprocPath
+                     ["--bib2yaml", "-f", drop 1 $ takeExtension fp]
+                     (UTF8.fromStringLazy bib)
+  let result = UTF8.toStringLazy result'
   if ec == ExitSuccess
      then do
-       let expectedDoc :: Maybe Aeson.Value
-           expectedDoc = Yaml.decode $ B.concat $ BL.toChunks expected
-       let resultDoc   :: Maybe Aeson.Value
-           resultDoc   = Yaml.decode $ B.concat $ BL.toChunks result
-       let result'   = BL.fromChunks [Yaml.encode resultDoc]
-       let expected' = BL.fromChunks [Yaml.encode expectedDoc]
-       if expected' == result'
+       if expected == result
           then err "PASSED" >> return Passed
           else do
             err $ "FAILED"
-            showDiff expected' result'
+            showDiff expected result
             return Failed
      else do
        err "ERROR"
